/**
 * \file mgui/dlgwizard.h
 * \brief MGUI::DLGWIZARD class definition.
 *
 * \if NODOC
 * $Id: dlgwizard.h_v 1.13 2012/01/27 18:32:00 mju Exp $
 *
 * $Log: dlgwizard.h_v $
 * Revision 1.13  2012/01/27 18:32:00  mju
 * Remove mfc stuff.
 *
 * Revision 1.12  2011/11/22 21:32:34  mju
 * Fix dox dependencies.
 *
 * Revision 1.1  2006/04/14 17:41:29  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_DLGWIZARD_H
#define	INC_MGUI_DLGWIZARD_H

#ifndef	INC_MGUI_SHELL_H
#include <mgui/shell.h>
#endif

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif


namespace MGUI {

class LAYOUT_WIZARD_PAGE;

//! Base class for wizard dialog support for modal behavior.
//!
//! Most derived classes will also override v_OnFinish
//! to perform the desired update of the external data when the user presses 'Finish'
//! When overriding the protected v_On...() virtual methods pay close attention to 
//! when the corresponding base method is to be called to ensure correct behavior.
class DLGWIZARD : public MGUI::SHELL {
	public:

		enum BTNFLAGS {
			BTNFLAG_None = 0x00,
			BTNFLAG_Next = 0x01,
			BTNFLAG_Back = 0x02,
			BTNFLAG_Finish = 0x04,
			BTNFLAG_DisabledFinish = 0x08
			};

		//! Dialog style settings, may be combined.
		enum DLGSTYLE {
			DLGSTYLE_Default =			0x0000,		//!< Default style
			DLGSTYLE_Resizable =			0x0001,		//!< Dialog is resizable
			};

		//! Destructor.
		virtual ~DLGWIZARD (
			) = 0;

		//! Open dialog in modal state and wait for user to dismiss via OK/Cancel.
		//! @return 0 if OK, EUserCancel if Cancel, < 0 if error
		ERRVALUE DoModal (
			MDLGPARENT dlgparent							//!< Parent for modal dialog
			);

		//! Get index to 'active' page.
		//! @return Zero-based index to active page, -1 if none created.
		int GetActiveIndex (
			) const;

		//! Get number of pages currently in the wizard
		//! @return Number of pages
		int GetNumPages (
			) const;

		//! Set active page by index.
		void SetActiveIndex (
			int index										//!< Zero-based page index
			);
			
		//! Set back / next / finish button status
		//! Specifying the flag enables.  Leaving out the flag disables
		void SetWizardButtons (
			BTNFLAGS flags
			);

	protected:

		//! Constructor, initializes only.
		DLGWIZARD (
			const MISTRING& title,						//!< Title string, can use TEXTID also
			HELPID HelpID,									//!< Help ID, use HELPID__None if no 'help' available
			DLGSTYLE style = DLGSTYLE_Default		//!< Dialog style settings
			);
			
		//! Send 'Cancel' notification to dialog.
		void NotifyCancel (
			) { v_OnCancel(); }

		//! Set value to return from dialog.
		void SetReturnValue (
			int RetValue
			) { m_RetValue = RetValue; }

		//! Set margin for root pane.
		//! This must be called before DoModal() and is usually done in
		//! the subclass constructor.
		void SetRootMargin (
			int RootMargin
			) { m_RootMargin = RootMargin; }

		//! Set spacing for root pane.
		//! This must be called before DoModal() and is usually done in
		//! the subclass constructor.
		void SetRootSpacing (
			int RootSpacing
			) { m_RootSpacing = RootSpacing; }

		//! Set dialog style.
		//! This must be called before DoModal() and is usually done in
		//! the subclass constructor.
		void SetStyle (
			DLGSTYLE style
			) { m_Style = style; }

		// Overridables that may be called from derived classes.

		//! Called when 'Cancel' button is pressed.
		//! Derived class must call base class v_OnCancel() method AFTER performing its own processing
		//! to actually close the dialog.  In some cases the user may be given an opportunity to 
		//! verify that they intended to press the Cancel button.  If they elect not to cancel then
		//! the base class v_OnCancel() method should not be called.  This will cause the dialog to
		//! remain open and allow the user to retain the settings.
		//! Derived classes should generally declare this method as private.
		virtual void v_OnCancel ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLGWIZARD THISCLASS;

		DLGSTYLE m_Style;
		ERRVALUE m_RetValue;
		int m_RootSpacing;
		int m_RootMargin;

		SIMPLE_ARRAY<LAYOUT_WIZARD_PAGE*> m_PagePtrs;
		int m_ActiveIndex;
		LAYOUT_PANE m_ButtonPane;
		CTRL_PUSHBUTTON m_BtnBack;
		CTRL_PUSHBUTTON m_BtnNext;
		CTRL_PUSHBUTTON m_BtnFinish;
		CTRL_PUSHBUTTON m_BtnCancel;
		CTRL_PUSHBUTTON m_BtnHelp;

		//! Create actual wizard dialog and root pane.
		void Close (bool notify = true);

		ERRVALUE CreateDlg (
			MDLGPARENT dlgparent
			);

		void OnCmdBack ();
		void OnCmdFinish ();
		void OnCmdNext ();

		DLGWIZARD (const DLGWIZARD&);
		DLGWIZARD& operator= (const DLGWIZARD&);

		// Overrides from SHELL.
		void v_Destroy ();
		int v_CreateContent ();
		void v_OnCloseRequest ();

		friend class LAYOUT_WIZARD_PAGE;
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Called when dialog gets closed.
		virtual void v_OnClose ();

		//! Called when dialog is destroyed.
		virtual void v_OnDestroy ();

		//! Called after dialog is actually opened.
		virtual void v_OnOpen ();

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLGWIZARD::DLGSTYLE)
DEFINE_ENUM_OP_BITWISE(DLGWIZARD::BTNFLAGS)
#endif


class LAYOUT_WIZARD_PAGE : public MGUI::LAYOUT_PANE_ROOT {
	public:

		enum PAGE {
			PAGE_Same = -2,
			PAGE_Next = -1,
			PAGE_Back = PAGE_Next
			};

		//! Constructor.
		LAYOUT_WIZARD_PAGE (
			);

		//! Destructor.
		virtual ~LAYOUT_WIZARD_PAGE (
			);

		//! Create page with tab label from MISTRING.
		//! @return Page index.
		int Create (
			MGUI::DLGWIZARD& wizard,					//!< Wizard to add page to
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Vertical
			);

		//! Get the index of 'this' page
		//! @return 'this' wizard page index
		int GetPageIndex (
			) const { return (m_index); }

		//! Determine if this is the 'active' page.
		bool IsActive (
			) const;

		//! Set whether page is 'enabled' or not.
		void SetEnabled (
			bool enabled = true
			);

		//! Set next / finish button status.
		void SetNextEnabled (
			bool enabled,						//!< Whether button is enabled or not
			bool ForceToFinish = false		//!< Force to display "Finish" instead of "Next" even if not last page
			);

	protected:

		//! Set to ignore nonvisible items when doing layout.
		//! This must be called before Create() and is usually done in the subclass constructor.
		void SetIgnoreNonVisible (
			bool IgnoreNonVisible = true
			) { m_IgnoreNonVisible = IgnoreNonVisible; }

		//! Set whether to not automatically add "expanding space" at end (bottom usually) of page.
		//! This must be called before Create() and is usually done in the subclass constructor.
		//! Default behavior is to add expanding space.
		void SetNoSpaceExpanding (
			bool NoSpaceExpanding = true
			) { m_NoSpaceExpanding = NoSpaceExpanding; }

		//! Called when 'Finish' button is pressed.
		//! Derived class must call base class v_OnWizardFinish() method AFTER performing its own processing
		//! to actually close the dialog.  If the dialog settings do not pass their associated
		//! validation test then an appropriate message should be displayed for the user and
		//! the base class v_OnFinish() method should not be called.  This will cause the dialog to
		//! remain open and allow the user to correct the settings.
		//! Derived classes should typically declare this method as private.
		virtual void v_OnWizardFinish ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DLGWIZARD *m_pWizard;
		MGUI::LAYOUT_ORIENTATION m_Orientation;
		int m_index;
		bool m_IgnoreNonVisible;
		bool m_NoSpaceExpanding;

		LAYOUT_WIZARD_PAGE (const LAYOUT_WIZARD_PAGE&);
		LAYOUT_WIZARD_PAGE& operator= (const LAYOUT_WIZARD_PAGE&);
		
		void InitPageX (Widget form);

		friend class DLGWIZARD;
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Create page contents.
		//! Derived class MUST implement this.
		virtual void v_OnInitPage () = 0;

		//! Called when page is made active.
		virtual void v_OnSetActive ();

		//! Called when page is made inactive.
		virtual void v_OnSetInactive ();

		//! Called when the 'Back' button is pressed.
		//! @return PAGE_Back to automatically advance to the previous page; PAGE_Same to prevent the page from changing.
		//! To jump to a page other than the previous one, return the index of the dialog to be displayed.
		//! Default implementation returns PAGE_Back.
		virtual int v_OnWizardBack ();

		//! Called when the 'Next' button is pressed.
		//! @return PAGE_Next to automatically advance to the next page; PAGE_Same to prevent the page from changing.
		//! To jump to a page other than the next one, return the index of the page to be displayed.
		//! Default implementation returns PAGE_Next.
		virtual int v_OnWizardNext ();

	};


}	// End namespace MGUI


#endif	// INC_MGUI_DLGWIZARD_H
