/**
 * \file mgui/editsml.h
 * \brief SML / Query / Script Editor class.
 *
 * \if NODOC
 * $Log: editsml.h_v $
 * Revision 1.20  2011/11/23 13:55:13  mju
 * Remove redundant hdr.
 *
 * Revision 1.19  2011/10/05 23:43:00  bcolombe
 * Add Run for standalone
 *
 * Revision 1.18  2011/09/20 16:09:00  mju
 * Add attachStatusBar.
 * Remove unused mfc methods.
 *
 * Revision 1.17  2011/09/01 18:00:26  mju
 * Exclude macro from dox.
 *
 * Revision 1.16  2011/01/18 16:25:59  bcolombe
 * *** empty log message ***
 *
 * Revision 1.15  2011/01/18 00:05:29  bcolombe
 * Allow using encrypted scripts
 *
 * Revision 1.14  2008/10/15 20:27:49  scowan
 * Forward declare objitem.
 *
 * Revision 1.13  2008/10/15 13:25:04  mju
 * Use smlbase hdr instead of sml hdr.
 *
 * Revision 1.12  2007/02/05 21:42:33  dwilliss
 * Moved defn of FORM_EDIT_SML_PARAMETERS to its own include file
 * (mgui/editsmlparameter.h)
 *
 * Revision 1.11  2007/02/02 23:22:25  dwilliss
 * first draft of form edit sml parameters.  Will eventually go in its
 * own include file
 *
 * Revision 1.10  2007/02/02 18:30:01  dwilliss
 * Added a flag to enable the parameters tab
 *
 * Revision 1.9  2007/02/01 23:13:02  dwilliss
 * changed GetScript method to return by value instead of by reference.
 *
 * Revision 1.8  2007/01/31 17:23:52  dwilliss
 * Added ability to specify SMLUSAGE on form and dialog
 *
 * Revision 1.5  2006/10/20 17:24:45  dwilliss
 * Added DLG_EDIT_SML
 *
 * Revision 1.1  2002/08/14 20:48:23  dwilliss
 * Initial revision
 * \endif
 **/

#ifndef INC_MGUI_EDITSML_H
#define INC_MGUI_EDITSML_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
#include <mi32/smlbase.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
namespace RVC {
	class OBJITEM;
	}
namespace MGUI {
	class CTRL_EDIT_TEXT;
	}
#endif


namespace MGUI {

//=====================================================================================================================
//! Form to edit SML scripts and queries.
class FORM_EDIT_SML : public MGUI::FORM_COMPOSITE {
	public:

		enum FLAGS {
			FLAG_Default =					0,			//!< No flags
			FLAG_AllowSaveEncrypted =	0x0001,	//!< Add the Save as / Ecrypted object option (disabled if running TNTlite)
			FLAG_EditParameters =		0x0002,	//!< Can edit script parameters
			FLAG_AllowLoadEncrypted =	0x0004	//!< can load encrypted script, need to make sure script returned is not serialized
			};

		//! Default constructor, initializes only.
		FORM_EDIT_SML ();

		//! Destructor.
		virtual ~FORM_EDIT_SML ();

		//! Attach statusbar for reporting messages.
		void AttachStatusBar (
			CTRL_STATUSBAR& StatusBar		//!< Status bar to attach, must exist for life of form
			);

		//! Check to see if the syntax of the current query is valid.
		//! Returns true if valid, false if not.
		bool CheckSyntax (
			bool bShowError = true			//!< If syntax invalid, display error if true
			);

		//! Run script
		void Run ();

		//! Create the form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			SML::USAGE usage,
			int width = 30,					//!< Width of editor part of form in 'em' characters
			int height = 10,					//!< Height of editor part of form in characters
			FLAGS flags = FLAG_Default,
			MGUI::LAYOUT_SIZEALIGN sizealign= MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Get CTRL_EDIT_TEXT of the editor
		MGUI::CTRL_EDIT_TEXT& GetEditText (
			);

		//! Get the current script string.
		const MISTRING GetScript (
			);

		const MISTRING& GetDisplayString (
			);

		//! Get the OBJITEM of the script.
		//! Note that if the script came from a .sml or .qry file, the objitem 
		//!  will return true from IsFileOnly().  If it returns false from
		//!  IsAssigned() then the script has not been saved or was not loaded
		//!  from a file or object.
		const RVC::OBJITEM& GetScriptObjItem (
			) const;

		//! Get toolbar pane, usually to add additional buttons.
		MGUI::LAYOUT_PANE& GetToolBarPane (
			);

		//! Returns true if the query has changed since it was last set.
		bool HasChanged (
			) const;

		//! Determine if encrypted
		bool IsEncrypted (
			) const;

		//! Load a script from a given object
		ERRVALUE LoadScript (
			const RVC::OBJITEM& ObjItem
			);

		//! Load a script from a given file
		ERRVALUE LoadScript (
			const FILEPATH& filepath,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8
			);

		//! Save the script to a given object
		ERRVALUE SaveScript (
			const RVC::OBJITEM& ObjItem
			);

		//! Save the script to a given file
		ERRVALUE SaveScript (
			const FILEPATH& filepath,
			CHAR_ENCODING Encoding = CHAR_ENCODING_UTF8
			);

		//! Set the SML::CONTEXT for this editor.
		//!	The following QFLAGs are implemented (defined in sml.h)
		//!		QFLAG_FileExtnSML		Make Open/Save/Save As buttons expect .sml files by default (normally expects .qry)
		void SetContext (
			SML::CONTEXT* context,
			int segment = 0,
			UINT32 flags = 0		//!< QFLAG_ constants from sml.h (QFLAG_FileExtnSML)
			);

		//! Set the script string.
		void SetScript (
			const MISTRING& script
			);

		//! Set the SML::USAGE for the script being edited.
		void SetUsage (
			SML::USAGE usage
			);

		//! Set whether or not to use syntax highlighting (default)
		//! Syntax highlighting is enabled by default
		void SetSyntaxHighlighting (
			bool bValue
			);

		//! Set a delegate to call when the script is modified
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS OnChange
			) { m_DelegateOnChange = OnChange; }

		//! Set a delegate to call when the script is loaded or saved
		void SetDelegateOnFileChanged (
			DELEGATE_VOID_NOPARMS OnFileChange
			) { m_DelegateOnFileChange = OnFileChange; }

		//! Set a delegate to call to update a status line
		void SetDelegateOnStatus (
			DELEGATE_VOID_MISTRING OnStatus
			) { m_DelegateOnStatus = OnStatus; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_Priv;
		friend class PRIV;

		DELEGATE_VOID_NOPARMS m_DelegateOnChange;
		DELEGATE_VOID_NOPARMS m_DelegateOnFileChange;
		DELEGATE_VOID_MISTRING m_DelegateOnStatus;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_EDIT_SML::FLAGS)
#endif

//=====================================================================================================================
//! Dialog for editing an SML script.  This is meant to be a modal dialog.
class DLG_EDIT_SML : public MGUI::DLGSHELL {
	public:

		//! Create a dialog for editing any SML script.  
		//! Call SetScript() before DoModal() and GetScript() after DoModal()
		DLG_EDIT_SML (
			const MISTRING& title,
			SML::CONTEXT* context,
			SML::USAGE usage,
			INT32 segment = 0,
			FORM_EDIT_SML::FLAGS flags = FORM_EDIT_SML::FLAG_Default
			);

		//! Create a dialog for editing any SML script.  
		//! Call SetScript() before DoModal() and GetScript() after DoModal()
		DLG_EDIT_SML (
			const MISTRING& title,
			SML::CONTEXT* context,
			INT32 segment = 0,
			FORM_EDIT_SML::FLAGS flags = FORM_EDIT_SML::FLAG_Default
			);

		virtual ~DLG_EDIT_SML ();

		const MISTRING& GetScript (
			) const;

		const MISTRING& GetDisplayString (
			);

		const RVC::OBJITEM& GetScriptObjItem (
			) const;

		bool IsEncrypted (
			) const;

		void SetScript (
			const MISTRING& script
			);

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_priv;

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();
#endif
	};

//=====================================================================================================================

}	// End namespace MGUI


#endif	// INC_MGUI_EDITSML_H

