/**
 * \file mgui/listbox.h
 * \brief MGUI::CTRL_LISTBOX definitions.
 *
 * \if NODOC
 * $Id: listbox.h_v 1.21 2011/11/09 15:56:03 mju Exp $
 *
 * $Log: listbox.h_v $
 * Revision 1.21  2011/11/09 15:56:03  mju
 * Remove mfc defns.
 *
 * Revision 1.12  2004/11/10 01:46:12  dfriberg
 * Initialize double-click function pointer in convenience class.
 *
 * Revision 1.11  2004/11/08 18:41:23  dwilliss
 * Added ExpandIfNeeded style
 *
 * Revision 1.10  2004/11/05 23:46:19  dwilliss
 * Added a STYLE value for ResizeIfPossible
 * Template version now lets you set OnDoubleClick
 *
 * Revision 1.1  2003/01/16 21:12:50  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_LISTBOX_H
#define	INC_MGUI_LISTBOX_H

#ifndef INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef INC_MI32_INIDEFNS_H
#include <mi32/inidefns.h>
#endif

namespace MGUI {

#ifndef GENERATING_DOXYGEN_OUTPUT
class XMLIST;
#endif

//===================================================================================================================
//! Display simple list of items which user can view and select.
//! In a single-selection list box, the user can select only one item.
//! In a multiple-selection list box, a range of items can be selected.
class CTRL_LISTBOX : public MGUI::CTRL {
	public:

		//! Listbox Style used by Create().
		enum STYLE {
			STYLE_Default =			0x00,
			STYLE_Sorted =				0x01,
			STYLE_MultiSelect =		0x02,			//!< Multiple items selectable by simple toggle of each item
			STYLE_ExtendedSelect =	0x04,			//!< Multiple items selectable by SHIFT/CTRL key and mouse.
			STYLE_ResizeIfPossible = 0x08,			//!< Try to expand and shrink to fit widest item. (uses scrollbar if it can't resize)
			STYLE_ExpandIfNeeded = 	0x10			//!< Try to expand fit widest item. (uses scrollbar if it can't resize)
			};

		//! Constructor.
		CTRL_LISTBOX ();

		//! Destructor.
		virtual ~CTRL_LISTBOX ();

		//! Add item to list box with MISTRING or TEXTID.
		//! If list box was created with STYLE_Sorted then the list will be resorted after the item
		//! is added, otherwise the item will be added to the end of the list.
		//! @return Zero-based index of item added or error < 0.
		int AddItem (
			ID id,											//!< ID to attach to item
			const MISTRING& string,						//!< String to show in list
			bool resize = true							//!< Attempt to resize if needed to fit new item string
			);

		//! Create the list box control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int height = 5,								//!< Height in characters
			int width = 0,									//!< Width in 'typical' characters
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand,
			STYLE style = STYLE_Default
			);

		//! Delete all items from list box.
		void DeleteAllItems ();

		//! Delete item from list box.
		void DeleteItem (
			int index										//!< Zero-based index of list box item to delete
			);

		//! Delete item from list box given ID.
		void DeleteItemID (
			ID id												//!< ID of item to delete
			);

		//! Deselect all items in multiple-selection list box.
		//! This method should not be used on single-selection list boxes.
		void DeselectAllItems ();

		//! Retrieve number of items in the list box.
		int GetCount () const;

		//! Get index to item from ID.
		//! @return Zero-based index or -1 if no matching item found.
		int GetIndexFromID (
			ID id
			) const;

		//! Get ID associated with list box item.
		//! @return ID specified when item was added.
		ID GetItemID (
			int index									//!< Zero-based index of list box item
			) const;

		//! Get text string associated with list box item.
		//! @return String containing text, will be empty if invalid item.
		MISTRING GetItemString (
			int index									//!< Zero-based index of list box item
			) const;

		//! Get number of selected items in multiple-selection list box.
		//! @return Number of selected items or 0 if none selected.
		int GetSelectedCount () const;

		//! Get currently selected item in single-selection list box.
		//! @return Zero-based index of selected item or -1 if none selected.
		int GetSelectedItem () const;

		//! Get ID of currently selected item in single-selection list box.
		//! @return ID or -1 if none selected.
		ID GetSelectedItemID () const;

		//! Retrieve indices of selected items in multiple-selection list box.
		ERRVALUE GetSelectedItems (
			SIMPLE_ARRAY<int>& items
			) const;

		//! Retrieve zero-based index of first visible item in list box.
		int GetTopIndex () const;

		//! Read setting from INI file.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			bool notify = true						//!< Call OnSelection() if control has already been created
			);

		//! Write currently selected item ID to INI file.
		void IniWrite (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField						//!< INI field to read from
			) const { ::IniWrite(IniHandle,IniGroup,IniField,static_cast<INT32>(GetSelectedItemID())); }

		//! Insert item at specified position in list box.
		//! Unlike the AddItem method this does not cause the list to be resorted.
		//! @return Zero-based index of item added or error < 0.
		int InsertItem (
			int index,									//!< Zero-based index of position at which item should be inserted
			ID id,										//!< ID to attach to item
			const MISTRING& string					//!< String to show in list
			);

		//! Determine if specified item in list box is selected.
		bool IsItemSelected (
			int index									//!< Zero-based index of list box item
			) const;

		//! Set delegate to call when user changes selected items.
		void SetDelegateOnChangeSelection (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeSelection = delegate; }

		//! Set delegate to call when user double-clicks on item.
		void SetDelegateOnDoubleClick (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnDoubleClick = delegate; }

		//! Select or deselect item in multiple-selection list box.
		//! This method should not be used on single-selection list boxes.
		void SetItemSelected (
			int index,									//!< Zero-based index of list box item.
			bool select = true						//!< Specifies how to set the selection, true to select, false to deselect.
			);

		//! Select item in single-selection list box.
		//! If item is not visible the list box will be scrolled to make it visible.
		//! This method should not be used on multiple-selection list boxes.
		void SetSelectedItem (
			int index									//!< Zero-based index of list box item, -1 to have no item selected
			);

		//! Select item in list box using item ID.
		//! If item does not exist then no item will be selected.
		//! If item is not visible the list box will be scrolled to make it visible.
		void SetSelectedItemID (
			ID id											//!< ID of item to select
			);

		//! Scroll list box until item specified by index is at the top or maximum scroll range reached.
		void SetTopIndex (
			int index									//!< Zero-based index of list box item
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE_VOID_NOPARMS m_DelegateOnChangeSelection;
		DELEGATE_VOID_NOPARMS m_DelegateOnDoubleClick;

		XMLIST *m_pxmlist;
		static void CB_ChangeSelection (Widget, CTRL_LISTBOX*, void*);
		static void CB_DoubleClick (Widget, CTRL_LISTBOX*, void*);

		void OnChangeSelection ();
		void OnDoubleClick ();

		CTRL_LISTBOX (const CTRL_LISTBOX&);
		CTRL_LISTBOX& operator= (const CTRL_LISTBOX&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::CTRL_LISTBOX::STYLE)
#endif

//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_LISTBOX_H
