/**
 * \file mgui/topshell.h
 * \brief MGUI::TOPSHELL definitions.
 *
 * \if NODOC
 * $Id: topshell.h_v 1.20 2012/01/27 19:06:06 mju Exp $
 *
 * $Log: topshell.h_v $
 * Revision 1.20  2012/01/27 19:06:06  mju
 * Rmv redundant hdr.
 *
 * Revision 1.19  2011/11/22 13:43:25  mju
 * Remove mfc stuff.
 *
 * Revision 1.18  2011/09/01 17:37:05  mju
 * Exclude macro from docs.
 *
 * Revision 1.17  2011/05/26 14:08:49  mju
 * Add documentation and remove unused method.
 *
 * Revision 1.16  2010/10/21 12:53:48  mju
 * Add isVisible.
 *
 * Revision 1.15  2008/11/21 23:13:44  bcolombe
 * Added RunJob button
 *
 * Revision 1.14  2008/11/07 20:54:37  mju
 * Add button_SaveJob and v_onSaveJob.
 *
 * Revision 1.11  2007/02/27 19:03:25  dwilliss
 * Added v_OnRun and methods for creating default buttons
 *
 * Revision 1.9  2005/12/07 14:24:52  mju
 * Add border parameter to setRootSpacing.
 *
 * Revision 1.4  2004/03/16 17:54:38  mju
 * Don't allow further override of v_OnDestroy or v_GetMainPane.
 *
 * Revision 1.1  2003/05/28 16:08:43  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_TOPSHELL_H
#define	INC_MGUI_TOPSHELL_H

#ifndef	INC_MGUI_SHELLBASE_H
#include <mgui/shell.h>
#endif

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif


namespace MGUI {

enum DOCKEDGE {
	DOCKEDGE_Left =			0x1000,
	DOCKEDGE_Top =				0x2000,
	DOCKEDGE_Right =			0x4000,
	DOCKEDGE_Bottom =			0x8000,
	DOCKEDGE_Any =				0xF000
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DOCKEDGE)
#endif


//! Base class for 'top level' application shells.
//! Top-level shells are windows whose parent is the 'desktop'.
class TOPSHELL : public MGUI::SHELL {
	public:

		enum BUTTONS {
			BUTTON_RunJob =		0x02,		//!< Call v_OnRunJob when pressed
			BUTTON_SaveJob =		0x04,		//!< Call v_OnSaveJob when pressed
			BUTTON_Close =			0x08,		//!< Calls v_OnCloseRequest() when pressed
			BUTTON_Help =			0x10,		//!< Displays the help specified by the HELPID in the constructor when pressed
			BUTTON_Run =			0x20,		//!< Calls v_OnRun when pressed
			BUTTON_Exit =			0x40,		//!< Same as BUTTON_Close, except for the label.  Note: if you want the button to actually exit the application, you should call APP_MAIN::Exit() from v_OnCloseRequest().
			BUTTONS_Default =		0x00		//!< Creates 'Run', 'Exit' and 'Help' buttons
			};

		enum STYLE {
			STYLE_Default =		0x0000,
			STYLE_NoClose =		0x0001,
			STYLE_NoMinimize =	0x0002,
			STYLE_NoMaximize =	0x0004,
			STYLE_NoResize =		0x0008,		//!< Don't allow resizing, implies NoMaximize also
			STYLE_RightAlignButtons = 0x0010,
			};

		//! Constructor.
		TOPSHELL (
			HELPID helpid
			);

		//! Destructor.
		virtual ~TOPSHELL ();

		//! Create the actual shell graphical object.
		void Create (
			const char *PlacementKey = 0,					//!< Ini/Registry key for automatic placement management
			TEXTID TitleID = TEXTID__None,				//!< Text ID for title
			STYLE style = TOPSHELL::STYLE_Default,		//!< Shell style
			ICONID WindowIcon = ICONID_TITLE_TNTMIPS	//!< Icon to place at the top left of the window title bar
			);

		//! Enable edges of shell for control bar docking.
		//! If not specified and control bars are created for the shell then all edges are
		//! assumed to be enabled for docking.
		void EnableDocking (
			DOCKEDGE dockedges = DOCKEDGE_Any			//!< Sides of shell enabled for docking of control bars
			);

		//! Determine if currently visible.
		bool IsVisible () const;

		//! Set whether shell is visible or not.
		//! Will call v_OnOpen or v_OnClose.
		void SetVisible (
			bool visible = true
			);

	protected:

		//! Create standard button with default label.
		//! @return Reference to button created (usually for alignment only).
		MGUI::CTRL_PUSHBUTTON& CreateButton (
			BUTTONS button									//!< Button to create
			);

		//! Create standard button with label override.
		//! @return Reference to button created (usually for alignment only).
		MGUI::CTRL_PUSHBUTTON& CreateButton (
			BUTTONS button,								//!< Button to create
			const MISTRING& label
			);

		//! Create application-defined button at the bottom of the window. 
		//! If you want, for example 'Run', 'Test', 'Exit' and 'Help' buttons,
		//! you can use CreateButton for the standard ones and AddButton for
		//! the 'Test' button.  The order you create the buttons is important.
		//! The Button parameter should be a reference to a CTRL_PUSHBUTTON which
		//! is a member of your TOPSHELL-derived class.
		//! This method will take care of making all the buttons the same size.
		void CreateButton (
			MGUI::CTRL_PUSHBUTTON& Button,
			const MISTRING& label
			);

		//! Create standard buttons at bottom of dialog.
		//! Default button labels will be 'Run...', 'Exit', and 'Help'
		void CreateButtons (
			BUTTONS buttons = BUTTONS_Default		//!< Buttons to create
			);

		//! Set to ignore nonvisible items when doing layout.
		//! This must be called before Create() and is usually done in
		//! the subclass constructor.
		void SetIgnoreNonVisible (
			bool IgnoreNonVisible = true
			) { m_IgnoreNonVisible = IgnoreNonVisible; return; }

		//! Set spacing and border for root pane.
		//! This must be called before Create() and is usually done in the
		//! subclass constructor.  If not called the default spacing is 4 pixels and border is 0.
		void SetRootSpacing (
			int RootSpacing,
			int RootBorder = 0
			) { m_RootSpacing = RootSpacing; m_RootBorder = RootBorder; }

		//! Enable (or disable) the Run, Run Job and Save Job buttons.
		void SetRunEnabled (
			bool enabled = true
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		Widget m_form;
		bool m_IgnoreNonVisible;
		int m_RootSpacing;
		int m_RootBorder;
		STYLE m_Style;

		LAYOUT_PANE m_ButtonPane;
		CTRL_PUSHBUTTON m_RunJobPB;
		CTRL_PUSHBUTTON m_SaveJobPB;
		CTRL_PUSHBUTTON m_RunPB;
		CTRL_PUSHBUTTON m_ExitPB;
		CTRL_PUSHBUTTON m_HelpPB;
		LAYOUT_EQUALIZER *m_Equalizer;	// Only allocated and used if custom buttons are added
		
		//! Get pane to create buttons at bottom of dialog.
		//! This allows additional 'non-standard' buttons to be added to the dialog.
		//! This will create the pane if it has not been created already.
		MGUI::LAYOUT_PANE& GetButtonPane (
			bool AddSpace = true							//!< Add space if creating so buttons placed at right
			);

		void EqualizeButtons ();

		// Overrides from SHELL.
		void v_Destroy ();

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables

		//! Create shell contents.
		//! Derived class MUST implement this.  Function must return TRUE to show shell immediately,
		//! FALSE to not show the shell, or an error code < 0.  This will be called by the shell
		//! creation code and thus must not be called directly.  This is normally overridden only by
		//! the specific shell implementation and not by the code that actually uses that shell implementation.
		//! The implementation should call GetMainPane() to determine the pane to create the controls in.
		virtual int v_CreateContent () = 0;

		//! Called when shell gets hidden or closed.
		virtual void v_OnClose ();

		//! Called when user requests that shell be 'closed'.
		//! Usually results from an OS or Window Manager notification.
		//! To honor request and close the shell, override and call Destroy().
		virtual void v_OnCloseRequest () = 0;

		//! Called after shell is actually displayed.
		virtual void v_OnOpen ();

		//! Called when the 'Run' button is pressed
		virtual void v_OnRun ();

		//! Called when the 'Run Job' button is pressed
		virtual void v_OnRunJob ();

		//! Called when the 'Save Job' button is pressed
		virtual void v_OnSaveJob ();
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(TOPSHELL::BUTTONS)
DEFINE_ENUM_OP_BITWISE(TOPSHELL::STYLE)
#endif

}	// End namespace MGUI

#endif	// INC_MGUI_TOPSHELL_H
