/**
 * \file mi32/jpegext.h
 * \brief JPEGEXT class definitions.
 *
 * \if NODOC
 * $Id: jpegext.h_v 1.19 2011/10/12 12:41:24 mju Exp $
 *
 * $Log: jpegext.h_v $
 * Revision 1.19  2011/10/12 12:41:24  mju
 * Remove redundant hdr.
 *
 * Revision 1.18  2007/01/04 21:54:29  dwilliss
 * Added GetMetadata and GetExifData methods
 *
 * Revision 1.17  2006/11/08 15:58:07  mju
 * Add setZoom.
 *
 * Revision 1.16  2006/11/06 18:49:29  mju
 * Make more stuff private so can more easily close/reopen for rewind.
 *
 * Revision 1.15  2006/11/06 14:42:17  mju
 * Add rewind method.
 *
 * Revision 1.14  2006/10/18 22:14:18  dfriberg
 * Documentation
 *
 * Revision 1.13  2006/02/01 22:25:00  dfriberg
 * Add method to get bytes written to JPEG
 *
 * Revision 1.12  2005/12/09 21:47:26  mju
 * Inclusion guard endif needs to be at end.
 *
 * Revision 1.11  2005/09/12 20:25:23  dfriberg
 * Class will ALWAYS take ownership of buffer passed to make or open methods
 *
 * Revision 1.10  2005/09/12 20:22:20  dfriberg
 * Correction, takes UINT8*&
 *
 * Revision 1.9  2005/09/12 20:02:48  dfriberg
 * Open takes UINT8*
 *
 * Revision 1.8  2005/09/09 22:11:23  dfriberg
 * Add JPEGEXT_MEM::GetBufferSize()
 *
 * Revision 1.7  2005/07/22 22:01:32  dfriberg
 * Minor API changes
 *
 * Revision 1.6  2005/07/21 19:38:01  dfriberg
 * Modified API, added JPEGEXT_MEM
 *
 * Revision 1.5  2005/07/19 22:14:20  dfriberg
 * Compile errors
 *
 * Revision 1.4  2005/07/19 16:02:49  dfriberg
 * Undef classlibexport
 *
 * Revision 1.3  2005/07/18 20:08:47  dfriberg
 * Added more methods
 *
 * Revision 1.2  2005/07/18 18:28:06  dfriberg
 * Compile errors
 *
 * Revision 1.1  2005/07/18 18:21:15  dfriberg
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_JPEGEXT_H
#define INC_MI32_JPEGEXT_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class GENERICRW;
class FILEPATH;
class MISTRING;
namespace Exiv2 {
	class ExifData;
	}
#endif


//! Base class for JPEG image access
class CLASSLIBEXPORT JPEGEXT {
	public:
		class MAKEPARMS;

		enum DCTMETHOD {
			DCT_ISLOW,									//! Slow but accurate integer algorithm
			DCT_IFAST,									//! faster, less accurate integer method
			DCT_FLOAT,									//! floating-point method
			DCT_DEFAULT,								//! default method (normally DCT_ISLOW)
			DCT_FASTEST									//! fastest method (normally DCT_IFAST)
			};

		enum COLORSPACE {
			CS_Grayscale,
			CS_RGB,
			CS_CMYK
			};

		//! Destructor
		virtual ~JPEGEXT ();

		//! Get the current size of the compressed JPEG if opened for writing.
		//! Do not use if opened for reading.
		INT64 GetCompressedSize (
			) const;

		//! Get the color space of an existing JPEG.
		COLORSPACE GetColorSpace (
			) const;

		//! Get the Exif data for an existing JPEG.
		//! Will return NULL if the Exif header could not be read.
		const Exiv2::ExifData* GetExifData (
			) const;

		//! Get the height of an existing JPEG.
		//! Result is affected by use of SetZoom.
		UINT32 GetHeight (
			) const;

		//! Get the Exif data for an existing JPEG as a formatted text.
		//! Does NOT clear the string first
		//! Returns true if there was Exif data, false if not
		bool GetMetadata (
			MISTRING& string
			);

		//! Get the number of components of an existing JPEG.
		UINT16 GetNumComponents (
			) const;

		//! Get the width of an existing JPEG.
		//! Result is affected by use of SetZoom.
		UINT32 GetWidth (
			) const;

		//! Read lines.
		//! Returns EBadFuncParm if this is opened for writing.
		ERRVALUE ReadLines (
			void* buffer,								//!< Buffer to contain pixels
			UINT32 numlines							//!< Number of scanlines to read
			);

		//! Rewind reader to beginning.
		ERRVALUE Rewind (
			);

		void SetNumComponents (
			UINT16 components
			);

		//! Set zoom value for decompression.
		//! Must be used after Open or Rewind before ReadLines.
		//! Will affect result of GetHeight and GetWidth
		ERRVALUE SetZoom (
			double zoom									//!< Zoom value, must be 1, 1/2, 1/4 or 1/8
			);

		ERRVALUE WriteLines (
			const void* buffer,						//!< Buffer containing pixels to write
			UINT32 numlines							//!< Number of lines to write
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		#endif

	protected:
		enum OPENMODE {
			OPENMODE_Read = 0,
			OPENMODE_Write = 1
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT
		PRIV* m_pPriv;
		#endif

		//! Base class, cannot be constructed directly.
		JPEGEXT (
			PRIV *pPriv
			);

	};

//! Class for handling JPEG files.
class CLASSLIBEXPORT JPEGEXT_FILE : public JPEGEXT {
	public:
		JPEGEXT_FILE (
			);

		virtual ~JPEGEXT_FILE (
			);

		ERRVALUE Close (
			);

		ERRVALUE Make (
			const FILEPATH& fp,						//!< Destination file path
			const MAKEPARMS& parms					//!< Parameters for making new JPEG file.
			);

		ERRVALUE Open (
			const FILEPATH& fp						//!< Source file path
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV_FILE;
		PRIV_FILE* m_pPrivFile;
		#endif
	};

//! Class for handling JPEG memory buffers.  
class CLASSLIBEXPORT JPEGEXT_MEM : public JPEGEXT {
	public:

		JPEGEXT_MEM (
			);

		virtual ~JPEGEXT_MEM (
			);

		ERRVALUE Close (
			);

		//! Get the internal buffer.
		const UINT8* GetBuffer (
			) const;

		//! Get the size of the internal buffer.  Warning - this is the internal buffer size, 
		//! which is not necessarily the size of the JPEG-compressed data.
		INT64 GetBufferSize (
			) const;

		//! Get the internal buffer and transfer ownership to caller (i.e. you must free it).
		UINT8* Detach (
			);

		//! Make a new in-memory JPEG buffer.
		ERRVALUE Make (
			const MAKEPARMS& parms
			);

		//! Create a JPEG buffer.  The caller does NOT retain ownership of the buffer passed.
		//! If the buffer is not sufficiently large to store the entire image,
		//! it will be resized.
		ERRVALUE Make (
			UINT8*& buf,
			UINT32 bufsize,
			const MAKEPARMS& parms
			);

		//! Open an existing JPEG buffer.
		ERRVALUE Open (
			UINT8*& buf,
			UINT32 bufsize
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV_MEM;
		PRIV_MEM* m_pPrivMem;
		#endif
	};


//! Parameters for making a new JPEG
class CLASSLIBEXPORT JPEGEXT::MAKEPARMS {
	public:

		//! Constructor.
		MAKEPARMS (
			UINT32 width = 0,
			UINT32 height = 0,
			UINT32 quality = 75
			);

		//! Set the algorithm used for the DCT step.
		void SetDCTMethod (
			DCTMETHOD method
			);

		//! Set the input color space.
		void SetColorSpace (
			COLORSPACE colorspace
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_Width;
		UINT32 m_Height;
		UINT32 m_Quality;
		DCTMETHOD m_DCTMethod;
		COLORSPACE m_ColorSpace;
		friend class JPEGEXT;
		friend class JPEGEXT::PRIV;
		#endif
	};

#undef CLASSLIBEXPORT

#endif	// INC_MI32_JPEGEXT_H
