/**
 * \file mi32/rastergen.h
 * \brief RASTERGEN class definitions
 *
 * \if NODOC
 * $Id: rastergen.h_v 1.1 2004/09/23 14:33:53 mju Exp $
 *
 * $Log: rastergen.h_v $
 * Revision 1.1  2004/09/23 14:33:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_RASTERGEN_H
#define	INC_MI32_RASTERGEN_H

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif


//=====================================================================================================================

//! Base for raster generation classes.
class RASTERGEN {
	public:

		//! Default constructor.
		RASTERGEN ();

		//! Destructor.
		virtual ~RASTERGEN ();

		//! Compute target raster.
		ERRVALUE Compute (
			RVC::OBJITEM& ObjItemTargetRast			//!< Location for target raster passed/returned
			) const { return (v_Compute(ObjItemTargetRast)); }

	private:

		//! Override to compute target raster.
		virtual ERRVALUE v_Compute (
			RVC::OBJITEM& ObjItemTargetRast			//!< Location for target raster passed/returned
			) const = 0;
	};


//=====================================================================================================================

//! Generate raster for viewshed.
class RASTERGEN_VIEWSHED : public RASTERGEN {
	public:

		//! Constructor.
		RASTERGEN_VIEWSHED (
			);

		//! Destructor.
		virtual ~RASTERGEN_VIEWSHED (
			);

		//! Get viewpoint coordinate reference system.
		const SPATREF::COORDREFSYS& GetViewPointCRS (
			) const { return (m_ViewPointCRS); }

		//! Set horizontal viewing angle range.
		void SetHorizontalAngleRange (
			const DOUBLE_RANGE& HorizontalRange	//!< Viewing angle range (0-360 degrees)
			) { m_HorizontalRange = HorizontalRange; }

		//! Set maximum view distance.
		void SetMaximumDistance (
			double distance
			) { m_MaxDistance = distance; }

		//! Set minimum number of visible viewpoints to be considered as in viewshed.
		//! Default is 1 if not set.
		void SetMinimumVisiblePoints (
			int MinVisiblePoints						//! Minimum number of points to be considered visible, 0 for all viewpoints
			) { m_MinVisiblePoints = MinVisiblePoints; }

		//! Set source raster.
		void SetSourceRaster (
			const RVC::OBJITEM& ObjItemSourceRast
			) { m_ObjItemSourceRast = ObjItemSourceRast; }

		//! Set whether to use ellipsoid curvature.
		void SetUseEllipsoidCurvature (
			bool UseCurvature
			) { m_UseEllipsoidCurvature = UseCurvature; }

		//! Set vertical viewing angle range.
		void SetVerticalAngleRange (
			const DOUBLE_RANGE& VerticalRange	//!< Viewing angle range (-90-90 degrees)
			) { m_VerticalRange = VerticalRange; }

		//! Set vertical scale for elevation raster cell values.
		void SetVerticalScale (
			double scale
			) { m_VerticalScale = scale; }

		//! Set viewpoint coordinate reference system.
		//! If not set or undefined then viewpoints must be in source raster cell coordinates.
		void SetViewPointCRS (
			const SPATREF::COORDREFSYS& ViewPointCRS
			) { m_ViewPointCRS = ViewPointCRS; }

		//! Set viewpoint height above terrain.
		void SetViewPointHeight (
			double height
			) { m_ViewPointHeight = height; }

		//! Set viewpoint positions.
		void SetViewPoints (
			const DPOINT2D *ViewPoints,			//!< View points
			int NumViewPoints							//!< Number of viewpoints
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::OBJITEM m_ObjItemSourceRast;
		SPATREF::COORDREFSYS m_ViewPointCRS;
		DOUBLE_ARRAY<DPOINT2D> m_ViewPoints;
		DOUBLE_RANGE m_HorizontalRange;
		DOUBLE_RANGE m_VerticalRange;
		double m_ViewPointHeight;
		double m_MaxDistance;
		double m_VerticalScale;
		int m_MinVisiblePoints;
		bool m_UseEllipsoidCurvature;

		ERRVALUE v_Compute (
			RVC::OBJITEM& ObjItemTargetRast
			) const;

		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_RASTERGEN_H
