/**
 * \file simplear.h <mi32/simplear.h>
 * \brief Definitions for SIMPLE_ARRAY template
 *
 * \if NODOC
 * $Id: simplear.h_v 1.65 2012/06/20 17:16:42 mju Exp $
 *
 * $Log: simplear.h_v $
 * Revision 1.65  2012/06/20 17:16:42  mju
 * Add extend/reserve/resizeWithItemSize.
 *
 * Revision 1.64  2012/01/24 15:07:57  mju
 * Fix locate to avoid wrap when subtract and don't state that return value useful if not found.
 *
 * Revision 1.63  2010/09/10 15:06:49  mju
 * Add extend method.
 *
 * Revision 1.62  2010/08/27 12:51:31  mju
 * Change op=/copy to not alloc more than need.
 *
 * Revision 1.61  2010/07/19 14:31:42  mju
 * Add setAll.
 *
 * Revision 1.60  2010/04/02 15:27:31  mju
 * Use 'unsigned' for all indexing.
 * Change locate() to return whether 'found' in reference parameter.
 *
 * Revision 1.59  2009/09/14 17:04:05  mju
 * Add exchangeItems.
 *
 * Revision 1.58  2009/09/04 14:56:06  mju
 * Add removeItems.
 *
 * Revision 1.57  2009/04/23 17:13:03  mju
 * Add clearItems with no parms.
 *
 * Revision 1.56  2009/04/03 16:28:48  mju
 * Add insert method.
 *
 * Revision 1.54  2008/01/22 14:33:37  mju
 * In copy ctor don't copy if rhs has no items.
 *
 * Revision 1.53  2007/08/24 21:00:51  mju
 * Add getItems.
 *
 * Revision 1.52  2007/04/20 22:30:21  dfriberg
 * Set num items to 0 in destructor
 *
 * Revision 1.50  2006/08/17 20:48:04  scowan
 * Copy ctor will now set the items ptr to null before copy.
 *
 * Revision 1.47  2006/01/26 15:48:18  mju
 * Use template method for array operators so not ambiguous.
 *
 * Revision 1.45  2005/05/19 16:55:34  mju
 * Add sortRange.
 *
 * Revision 1.44  2004/04/16 21:50:03  scowan
 * Added locate method.
 *
 * Revision 1.43  2004/04/01 21:33:44  mju
 * Add GetLast and RemoveLast methods.
 *
 * Revision 1.39  2003/07/25 20:58:42  mju
 * Add Assign() method, deprecate old SetItems().
 * Add SetItems variant which does not shrink array.
 *
 * Revision 1.21  2001/01/30 15:01:10  mju
 * Add Get/SetItem methods as array/cast operators confused if array type is pointer.
 *
 * Revision 1.16  2000/08/17 16:01:00  scowan
 * Added unsigned int versions of array operator methods, was causeing a
 * strange error to occur if an unsigned int was used to access a simple array
 * entry.
 *
 * Revision 1.11  2000/07/27 15:40:36  dwilliss
 * Wrote Attach()/Detach() methods
 *
 * Revision 1.1  2000/05/10 13:26:15  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_SIMPLEAR_H
#define	INC_MI32_SIMPLEAR_H

#ifndef	INC_MI32_MEMALLOC_H
#include <mi32/memalloc.h>		// For MmAlloc/Realloc/Free defns
#endif

#ifndef	INC_MI32_SORT_H
#include <mi32/sort.h>
#endif

#ifndef	INC_MI32_MEMBUF_H
#include <mi32/membuf.h>		// For SwapBytes() defn
#endif

#ifndef	INC_MEMORY_H
#ifndef GENERATING_DOXYGEN_OUTPUT
#include <memory.h>				// memcpy()
#define	INC_MEMORY_H
#endif
#endif

#ifndef	INC_STRING_H
#ifndef GENERATING_DOXYGEN_OUTPUT
#include <string.h>				// memmove/memset
#define	INC_STRING_H
#endif
#endif

//! Simple array template.
//!
//! This template is designed to contain an array of items which can be allocated
//! using MmAlloc/MmRealloc.  The primary purpose of this template is to replace
//! allocated arrays where direct access to both array elements and the ability
//! to access a pointer to the first element (as for a C array) is required.
//! Do not use this template with classes which require constructors or have
//! virtual methods or which do not have compiler-generated assignment operators.
//!
//! For maximum efficiency, only minimal validation of access to array elements is performed.
//! No initialization of array items is performed by the template.
//!
//! For doubles and objects based on doubles, use DOUBLE_ARRAY<> in mi32/doublear.h
//!
//! Instances of this class are passed to DLLs, so do not add or remove members.

template <class _CT> class SIMPLE_ARRAY {
	public:

		//! Default constructor, creates empty array.
		SIMPLE_ARRAY (
			):
			m_items(0),
			m_numitems(0),
			m_numalloc(0),
			m_RefCount(1)
			{ }

		//! Copy constructor.
		SIMPLE_ARRAY (
			const SIMPLE_ARRAY<_CT>& rhs
			):
			m_items(0),
			m_numitems(0),
			m_numalloc(0),
			m_RefCount(1)
			{
			if (rhs.m_numitems > 0) {
				MmAllocExc((void**)&m_items,rhs.m_numalloc*sizeof(_CT));
				m_numalloc = rhs.m_numalloc;
				memcpy(m_items,rhs.m_items,rhs.m_numitems*sizeof(_CT));
				m_numitems = rhs.m_numitems;
				}
			}

		//! Destructor.
		~SIMPLE_ARRAY (
			) {
			if (m_items != 0) {
				free(m_items);
				m_numitems = 0;
				}
			}

		//! Assignment, may throw exception if out of memory.
		//! Use Copy() method for error checking.
		SIMPLE_ARRAY<_CT>& operator= (
			const SIMPLE_ARRAY<_CT>& rhs
			) {
			if (this != &rhs) {
				if (m_numalloc < rhs.m_numitems) {
					Free();
					MmAllocExc((void**)&m_items,rhs.m_numitems*sizeof(_CT));
					m_numalloc = rhs.m_numitems;
					}
				memcpy(m_items,rhs.m_items,rhs.m_numitems*sizeof(_CT));
				m_numitems = rhs.m_numitems;
				}
			return (*this);
			}

		//! Equality operator.
		bool operator== (
			const SIMPLE_ARRAY<_CT>& rhs
			) const {
			if (m_numitems != rhs.m_numitems) return (false);
			if (m_numitems == 0) return (true);
			return (memcmp(m_items,rhs.m_items,m_numitems*sizeof(_CT)) == 0);
			}

		//! Inequality operator.
		bool operator!= (
			const SIMPLE_ARRAY<_CT>& rhs
			) const {
			return (!operator==(rhs));
			}

		//! Cast to const array of template type.
		operator const _CT*(
			) const {
			return (m_items);
			}

		//! Cast to array of template type.
		operator _CT*(
			) {
			return (m_items);
			}

		//! Read (const) access to array element.
		//! @note Does not verify that index is within valid range.
		template <typename _IDXTYPE> const _CT& operator[] (
			_IDXTYPE index
			) const {
			return (m_items[index]);
			}

		//! Write (non-const) access to array element.
		//! @note Does not verify that index is within valid range.
		template <typename _IDXTYPE> _CT& operator[] (
			_IDXTYPE index
			) {
			return (m_items[index]);
			}

		//! Increment reference count.
		void AddRef (
			) { ++m_RefCount; }

		//! Append item to end of array.
		ERRVALUE Append (
			const _CT& item
			) {
			if (m_numitems >= m_numalloc) {
				ERRVALUE err;
				if ((err = Reserve(MAX(16,2*m_numitems))) < 0) return (err);
				}
			m_items[m_numitems++] = item;
			return (0);
			}

		//! Append items from C array.
		ERRVALUE Append (
			const _CT *items,						//!< Items to append
			unsigned numitems							//!< Number of items to append
			) {
			ERRVALUE err;
			if ((err = Reserve(m_numitems+numitems)) < 0) return (err);
			memcpy(m_items+m_numitems,items,numitems*sizeof(_CT));
			m_numitems += numitems;
			return (0);
			}

		//! Append items from SIMPLE_ARRAY.
		ERRVALUE Append (
			const SIMPLE_ARRAY<_CT>& rhs
			) {
			ERRVALUE err;
			if ((err = Reserve(m_numitems+rhs.m_numitems)) < 0) return (err);
			memcpy(m_items+m_numitems,rhs.m_items,rhs.m_numitems*sizeof(_CT));
			m_numitems += rhs.m_numitems;
			return (0);
			}

		//! Append item to end of array, throwing exception if out of memory.
		void AppendExc (
			const _CT& item
			) {
			if (m_numitems >= m_numalloc) {
				ReserveExc(MAX(16,2*m_numitems));
				}
			m_items[m_numitems++] = item;
			return;
			}

		//! Append items from C array, throwing exception if out of memory.
		void AppendExc (
			const _CT *items,						//!< Items to append
			unsigned numitems							//!< Number of items to append
			) {
			ReserveExc(m_numitems+numitems);
			memcpy(m_items+m_numitems,items,numitems*sizeof(_CT));
			m_numitems += numitems;
			return;
			}

		//! Append items from SIMPLE_ARRAY, throwing exception if out of memory.
		void AppendExc (
			const SIMPLE_ARRAY<_CT>& rhs
			) {
			ReserveExc(m_numitems+rhs.m_numitems);
			memcpy(m_items+m_numitems,rhs.m_items,rhs.m_numitems*sizeof(_CT));
			m_numitems += rhs.m_numitems;
			return;
			}

		//! Append item to end of array if unique.
		ERRVALUE AppendUnique (
			const _CT& item
			) {
			unsigned j;
			for (j = 0; (j < m_numitems); ++j) {
				if (m_items[j] == item) break;
				}
			return ((j == m_numitems) ? Append(item) : 0);
			}

		//! Append item to end of array if unique, throwing exception if out of memory.
		void AppendUniqueExc (
			const _CT& item
			) {
			unsigned j;
			for (j = 0; (j < m_numitems); ++j) {
				if (m_items[j] == item) break;
				}
			if (j == m_numitems) AppendExc(item);
			return;
			}

		//! Assign from C array, resizing to the specified number of items.
		ERRVALUE Assign (
			const _CT *items,						//!< Items to copy to array
			unsigned numitems						//!< Number of items
			) {
			ERRVALUE err;
			if ((err = Reserve(numitems)) < 0) return (err);
			memcpy(m_items,items,numitems*sizeof(_CT));
			m_numitems = numitems;
			return (0);
			}

		//! Attach a buffer to the SIMPLE_ARRAY.
		//! After passing a buffer to Attach(), the SIMPLE_ARRAY "<b>owns</b>" the
		//! buffer and will free it in its destructor.  
		//! @param items Reference to a pointer to the templated data type.
		//! The reason it's a reference is that after recording the pointer,
		//! Attach() will set <i>your</i> pointer to 0.  This will prevent you
		//! from accidently freeing a pointer you no longer own.  If you want
		//! it back, call the Detach() method
		void Attach (
			_CT *& items,						
			unsigned numitems					//!< Number of items in array
			) {
			Free();
			m_items = items;
			m_numitems = m_numalloc = numitems;
			items = 0;
			return;
			}

		//! Clear the array, does not free resources.
		void Clear (
			) { m_numitems = 0; }

		//! Clear all items in the array
		void ClearItems (
			) { memset(m_items, 0, m_numitems * sizeof(_CT)); }

		//! Clear specified items in the array
		void ClearItems (
			unsigned start,
			unsigned end = -1						//!< Default to use number of items
			) {
			if (end >= m_numitems) end = m_numitems - 1;
			if (end >= start) {		// Start might be past end
				memset(m_items + start, 0, (end - start + 1) * sizeof(_CT));
				}
			return;
			}

		//! Copy the contents of another SIMPLE_ARRAY.
		//! Same as assignment, but returns an error code instead of throwing an exception.
		ERRVALUE Copy (
			const SIMPLE_ARRAY<_CT>& rhs
			) {
			if (this != &rhs) {
				if (m_numalloc < rhs.m_numitems) {
					Free();
					ERRVALUE err = MmAlloc((void**)&m_items,rhs.m_numitems*sizeof(_CT));
					if (err < 0) return (err);
					m_numalloc = rhs.m_numitems;
					}
				memcpy(m_items,rhs.m_items,rhs.m_numitems*sizeof(_CT));
				m_numitems = rhs.m_numitems;
				}
			return (0);
			}

		//! Detach the buffer from the SIMPLE_ARRAY.
		//!
		//! This turns ownership of the buffer over to the caller, who is then
		//! responsible for seeing that it gets disposed of.
		//! After calling Detach(), the SIMPLE_ARRAY will behave as if it had
		//! just been constructed.  In other words, it will be a NULL pointer
		//! pointing to 0 items.
		//! This means that if you want to know how many items are in the
		//! array returned, you must find out @{before} you call Detach()!
		_CT* Detach (
			) {
			_CT* ret = m_items;
			m_numalloc = m_numitems = 0;
			m_items = 0;
			return (ret);
			}

		//! Switch positions of two items.
		void ExchangeItems (
			unsigned index1,
			unsigned index2
			) {
			_CT temp = m_items[index1];
			m_items[index1] = m_items[index2];
			m_items[index2] = temp;
			return;
			}

		//! Extend array by one entry without setting contents.
		ERRVALUE Extend (
			) {
			if (m_numitems >= m_numalloc) {
				ERRVALUE err;
				if ((err = Reserve(MAX(16,2*m_numitems))) < 0) return (err);
				}
			++m_numitems;
			return (0);
			}

		//! Extend array by one entry without setting contents using item size.
		//! \internal
		ERRVALUE ExtendWithItemSize (
			unsigned itemsize
			) {
			if (m_numitems >= m_numalloc) {
				ERRVALUE err;
				if ((err = ReserveWithItemSize(MAX(16,2*m_numitems),itemsize)) < 0) return (err);
				}
			++m_numitems;
			return (0);
			}

		//! Free the array.
		void Free (
			) {
			m_numalloc = m_numitems = 0;
			MmFree(m_items);
			return;
			}

		//! Get array item, does not verify index range.
		const _CT& GetItem (
			unsigned index
			) const { return (m_items[index]); }

		//! Get const pointer to items.
		const _CT* GetItems (
			) const { return (m_items); }

		//! Get pointer to items.
		_CT* GetItems (
			) { return (m_items); }

		//! Get last item in array.
		//! Does not verify that array is non-empty.
		const _CT& GetLast (
			) const { return (m_items[m_numitems-1]); }

		//! Get maximum number of items in array.
		unsigned GetMaxItems (
			) const { return (m_numalloc); }

		//! Get number of items in array.
		unsigned GetNumItems (
			) const { return (m_numitems); }

		//! Get current reference count.
		UINT32 GetRefCount (
			) const { return (m_RefCount); }

		//! Get size of array required to hold current number of items in bytes.
		unsigned GetSizeInBytes (
			) const { return (m_numitems*sizeof(_CT)); }

		//! Determine if an item exists in the array.
		//! Item must have a valid operator==()
		//! @return 'True' if item found
		bool HasItem (
			const _CT& item
			) const {
			for (unsigned i = 0; (i < m_numitems); ++i) {
				if (m_items[i] == item) return (true);
				}
			return (false);
			}

		//! Insert item at specified position
		ERRVALUE Insert (
			unsigned pos,
			const _CT& item
			) {
			if (pos < 0 || pos > m_numitems) return (EBadFuncParm);
			if (m_numitems >= m_numalloc) {
				ERRVALUE err;
				if ((err = Reserve(MAX(16,2*m_numitems))) < 0) return (err);
				}
			if (pos < m_numitems) {
				memmove(&m_items[pos+1], &m_items[pos], (m_numitems - pos) * sizeof(_CT));
				}
			++m_numitems;
			m_items[pos] = item;
			return (0);
			}

		//! Insert multiple items into the array
		void InsertItems (
			unsigned start,
			unsigned num = 1
			) {
			ReserveExc(m_numitems+num);
			memmove(&m_items[start + num], &m_items[start], (m_numitems - start) * sizeof(_CT));
			m_numitems += num;
			return;
			}

		//! Determine if array is empty.
		bool IsEmpty (
			) const { return (GetNumItems() == 0); }

		//! Search sorted array for an entry or the place where the entry should be.
		//! _Pr is a functor that returns -1 if 1st item < 2nd, 1 if 1st > 2nd, 0 if 1st == 2nd
		//! 'item' is the first parameter passed to the _Pr functor.
		//! @return Position of item if found, indeterminant if not found.
		template<typename _Pr1> unsigned Locate (
			const _CT& item,
			_Pr1 _Pr,
			bool& bFound					//!< Returned as 'true' if item found, 'false' if not
			) const {
			bFound = false;
			unsigned l = 1, r = m_numitems;
			while (l <= r) {
				unsigned q = (l+r) / 2 - 1;
				int test = _Pr(item, m_items[q]);
				if (test == 0) { bFound = true; return (q); }
				if (test < 0) {
					r = q;
					}
				else {
					l = q + 2;
					}
				}
			return (m_numitems);
			}

		//! Decrement reference count, delete if zero.
		void Release (
			) { if (--m_RefCount == 0) delete this; }

		//! Remove duplicate entries in the list, assume not sorted		
		void RemoveDuplicates (
			) {
			for (unsigned i = 0; (i < m_numitems); ++i) {
				for (unsigned k = i+1; (k < m_numitems); ++k) {
					if (m_items[i] == m_items[k]) {
						m_items[k] = m_items[m_numitems-1];
						--m_numitems;
						--k;
						}
					}
				}
			return;
			}		

		//! Remove an item by placing the end item in its place.
		void RemoveFast (
			unsigned item
			) {
			if (item >= 0 && item < m_numitems) {
				--m_numitems;
				m_items[item] = m_items[m_numitems];
				}
			return;
			}

		//! Remove item at specified index, maintaining order.
		//! Note that this is inefficient for large arrays.
		void RemoveItem (
			unsigned item
			) {
			if (item < m_numitems && item >= 0) {
				--m_numitems;
				if (item < m_numitems) { memmove(&m_items[item], &m_items[item+1], (m_numitems - item) * sizeof(_CT)); }
				}
			return;
			}

		//! Remove items starting at specified index, maintaining order.
		//! Note that this is inefficient for large arrays.
		void RemoveItems (
			unsigned item,
			unsigned count
			) {
			if (item < m_numitems) {
				if (count < 0 || item + count > m_numitems) count = m_numitems - item;
				m_numitems -= count;
				if (item < m_numitems) { memmove(&m_items[item], &m_items[item+count], (m_numitems - item) * sizeof(_CT)); }
				}
			return;
			}

		//! Remove last item(s) from array.
		void RemoveLast (
			unsigned count = 1									//!< Number of items to remove
			) {
			if (count > m_numitems)
				m_numitems = 0;
			else
				m_numitems -= count;
			return;
			}

		//! Remove items matching the item passed in.  _CT must have a operator==() defined
		//! Maintains item order
		//! @return 'True' if one or more items was removed
		bool RemoveMatchingItems (
			const _CT& item
			) {
			unsigned i, num;
			for (num = i = 0; (i < m_numitems); ++i) {
				if (m_items[i] != item) {
					if (num < i) m_items[num] = m_items[i];		// Avoid expensive copy if not needed
					++num;
					}
				}
			bool retval = (m_numitems != num);
			m_numitems = num;
			return (retval);
			}

		//! Reserve space in array for up to specified number of items.
		ERRVALUE Reserve (
			unsigned newmaxitems,						//!< New maximum number of items
			bool clear = false							//!< Clear the array by resetting number of items to 0
			) {
			if (clear) m_numitems = 0;
			return (ReserveWithItemSize(newmaxitems,sizeof(_CT)));
			}

		//! Reserve space in array for up to specified number of items.
		//! Throws exception if out of memory.
		void ReserveExc (
			unsigned newmaxitems,						//!< New maximum number of items
			bool clear = false							//!< Clear the array by resetting number of items to 0
			) {
			if (clear) m_numitems = 0;
			if (newmaxitems > m_numalloc) {
				if (m_numitems == 0) {
					//! Free the old array to avoid expensive copy in reallocation
					MmFree(m_items);
					}
				MmReallocExc((void**)&m_items,newmaxitems*sizeof(_CT));
				m_numalloc = newmaxitems;
				}
			return;
			}

		//! Reserve space in array for up to specified number of items with item size.
		//! \internal
		ERRVALUE ReserveWithItemSize (
			unsigned newmaxitems,						//!< New maximum number of items
			unsigned itemsize								//!< Item size
			) {
			if (newmaxitems > m_numalloc) {
				ERRVALUE err;
				if (m_numitems == 0) {
					//! Free the old array to avoid expensive copy in reallocation
					MmFree(m_items);
					}
				if ((err = MmRealloc((void**)&m_items,newmaxitems*itemsize)) < 0) return (err);
				m_numalloc = newmaxitems;
				}
			return (0);
			}

		//! Resize array to specified number of items.
		//! If the new size is larger than the previous size and the array is
		//! not cleared the new array entries will have indeterminant values.
		ERRVALUE Resize (
			unsigned numitems,							//!< New number of items desired
			bool keepold = true,					//!< Keep old array entries
			bool clear = false					//!< Clear new array items to zeros
			) {
			ERRVALUE err;
			if ((err = Reserve(numitems,!keepold)) < 0) return (err);
			if (clear && numitems > m_numitems) memset(static_cast<void*>(m_items+m_numitems),0,(numitems-m_numitems)*sizeof(_CT));
			m_numitems = numitems;
			return (0);
			}

		//! Resize array to specified number of items with item size.
		//! If the new size is larger than the previous size and the array is
		//! not cleared the new array entries will have indeterminant values.
		//! \internal
		ERRVALUE ResizeWithItemSize (
			unsigned numitems,							//!< New number of items desired
			unsigned itemsize,							//!< Array item size
			bool clearnew = true							//!< Clear new array items to zeros
			) {
			ERRVALUE err;
			if ((err = ReserveWithItemSize(numitems,itemsize)) < 0) return (err);
			if (clearnew && numitems > m_numitems) memset(static_cast<void*>(m_items+m_numitems),0,(numitems-m_numitems)*itemsize);
			m_numitems = numitems;
			return (0);
			}

		//! Reverse order of items in array.
		void Reverse (
			) {
			for (unsigned i = 0; (i < m_numitems/2); ++i) {
				_CT temp(m_items[i]);
				m_items[i] = m_items[m_numitems-i-1];
				m_items[m_numitems-i-1] = temp;
				}
			}

		//! Set all items to specified value.
		void SetAll (
			const _CT& value
			) {
			for (unsigned i = 0; (i < m_numitems); ++i) {
				m_items[i] = value;
				}
			}

		//! Set single item, does not verify index range.
		void SetItem (
			unsigned index,							//!< Item index, must be in range from 0 to NumItems-1
			const _CT& item							//!< Item value to set
			) {
			m_items[index] = item;
			}

		//! Set array items from C array, expanding if needed.
		//! Will not reduce the number of items in the array.
		ERRVALUE SetItems (
			unsigned firstitem,						//!< First array item to replace
			const _CT *items,							//!< Items to copy to array
			unsigned numitems							//!< Number of items to copy
			) {
			ERRVALUE err;
			if ((err = Reserve(firstitem+numitems)) < 0) return (err);
			memcpy(m_items+firstitem,items,numitems*sizeof(_CT));
			if (m_numitems < firstitem + numitems) m_numitems = firstitem + numitems;
			return (0);
			}

		//! Sort the items in the array.
		//! CompFunc should return 1 if the first item sorts before the 2nd item
		//! Otherwise, return 0.
		void Sort (
			int (*CompFunc)(_CT*, _CT*, void*),
			void *data = 0
			) {
			// Compare function cannot return 'bool', must be 'int'.
			heapsort(m_items, m_numitems, sizeof(_CT), reinterpret_cast<int (*)(void*, void*, void*)>(CompFunc), data);
			}
			
		//! Sort the items in the array.
		//! CompFunc should return 1 if the first item sorts before the 2nd item
		//! Otherwise, return 0.
		void SortRange (
			int (*CompFunc)(_CT*, _CT*, void*),
			unsigned firstidx,
			unsigned numitems,
			void *data = 0
			) {
			if (numitems <= 0 || firstidx < 0 || firstidx + numitems > m_numitems) return;
			heapsort(m_items + firstidx, numitems, sizeof(_CT), reinterpret_cast<int (*)(void*, void*, void*)>(CompFunc), data);
			}
			
		//! Swap (exchange) two SIMPLE_ARRAYs.
		//! This exchanges the internal buffers so minimal copying is performed for efficiency.
		void Swap (
			SIMPLE_ARRAY<_CT>& rhs
			) {
			_CT *t_items = m_items;
			m_items = rhs.m_items;
			rhs.m_items = t_items;
			unsigned t_numitems = m_numitems;
			m_numitems = rhs.m_numitems;
			rhs.m_numitems = t_numitems;
			unsigned t_numalloc = m_numalloc;
			m_numalloc = rhs.m_numalloc;
			rhs.m_numalloc = t_numalloc;
			}

		//! Swap bytes in a SIMPLE_ARRAY, assume that there is an overload to swap a _CT
		void SwapBytes (
			) {
			for (unsigned i = 0; (i < m_numitems); ++i) ::SwapBytes(m_items[i]);
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		_CT *m_items;								// The actual array
		unsigned m_numitems;						// Number of valid items
		unsigned m_numalloc;						// Number of items allocated
		UINT32 m_RefCount;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#endif	// INC_MI32_SIMPLEAR_H
