/**
 * \file trans2d.h <mi32/trans2d.h>
 * \brief Definitions for 2-D transformations and macros
 *
 * \if NODOC
 * $Id: trans2d.h_v 1.36 2011/09/23 15:43:07 mju Exp $
 *
 * $Log: trans2d.h_v $
 * Revision 1.36  2011/09/23 15:43:07  mju
 * Move trans2dmodel defns to own hdr.
 *
 * Revision 1.35  2011/06/27 13:57:13  mju
 * Fix doc warning.
 *
 * Revision 1.34  2009/04/24 20:08:18  mju
 * Add findBestTransConformal2D.
 *
 * Revision 1.31  2003/12/18 17:49:48  vdronov
 * added TRANS2DMODEL_3D
 *
 * Revision 1.28  2001/05/22 14:17:03  mju
 * \endif
**/

#ifndef	INC_MI32_TRANS2D_H
#define	INC_MI32_TRANS2D_H

#ifndef INC_MI32_CTRLPT_H
#include <mi32/ctrlpt.h>
#endif

#ifndef	INC_MI32_TRANS2DMODEL_H
#include <mi32/trans2dmodel.h>
#endif


#ifdef GEOMDLL
	#define GEOMLIBEXPORT MI_DLLEXPORT
#else
	#define GEOMLIBEXPORT MI_DLLIMPORT
#endif


/*-------------------------------------------------------------------------------------------------*/
/*		Define macros for fastest possible transformation															*/
/*-------------------------------------------------------------------------------------------------*/

#define trans2dm(ix,iy,mat,ox,oy) { double _trans2dm_tx_, _trans2dm_ty_; _trans2dm_tx_ = (ix) * mat[0][0] + (iy) * mat[0][1] + mat[0][2]; \
	_trans2dm_ty_ = (ix) * mat[1][0] + (iy) * mat[1][1] + mat[1][2]; *(ox) = _trans2dm_tx_; *(oy) = _trans2dm_ty_; }

#define trans2dms(ix,iy,mat) { double _trans2dms_tx_, _trans2dms_ty_; _trans2dms_tx_ = (ix) * mat[0][0] + (iy) * mat[0][1] + mat[0][2]; \
	_trans2dms_ty_ = (ix) * mat[1][0] + (iy) * mat[1][1] + mat[1][2]; (ix) = _trans2dms_tx_; (iy) = _trans2dms_ty_; }


#define trans2dx(ix,iy,mat)  ((ix) * mat[0][0] + (iy) * mat[0][1] + mat[0][2])
#define trans2dy(ix,iy,mat)  ((ix) * mat[1][0] + (iy) * mat[1][1] + mat[1][2])


/*-------------------------------------------------------------------------------------------------*/
/*		Function prototypes																									*/
/*-------------------------------------------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

//! Disect a trans2d into component parts (scale, rotation, shear)
GEOMLIBEXPORT void DisectTrans2D (
	const MAT3X3 mat,							//!< Transformation matrix
	double *xscale,							//!< X scale passed / returned
	double *yscale,							//!< Y scale passed / returned
	double *rot,								//!< Rotation passed / returned
	double *shear								//!< Shear passed / returned
	);

//! Compute "best" 2-D affine transformation for set of CTRLPOINT's
GEOMLIBEXPORT int FindBestTrans2D (
	int NumPoints,								//!< Number of points
	const CTRLPOINT *cp,						//!< Set of CTRLPOINT's
	MAT3X3 forward,							//!< Forward transformation passed / returned
	MAT3X3 inverse								//!< Inverse transformation passed / returned
	);

//! Find "best" affine transformation given set of control points using CTRLPOINT array
GEOMLIBEXPORT int FindBestTrans2DA (
	int NumPoints,								//!< Number of points
	const CTRLPOINT *cp,						//!< Set of CTRLPOINT's
	MAT3X3 forward,							//!< Forward transformation passed / returned
	MAT3X3 inverse,							//!< Inverse transformation passed / returned
	UINT32 flags								//!< Flags
	);

//! Compute "best" 2-D affine transformation for set of CTRLPOINT3's
GEOMLIBEXPORT int FindBestTrans2D3 (
	int NumPoints,								//!< Number of points
	const CTRLPOINT3 *cp3,					//!< Set of CTRLPOINT3's
	MAT3X3 forward,							//!< Forward transformation passed / returned
	MAT3X3 inverse								//!< Inverse transformation passed / returned
	);

//! Find "best" affine transformation given set of control points using CTRLPOINT3 array
GEOMLIBEXPORT int FindBestTrans2D3A (
	int NumPoints,								//!< Number of points
	const CTRLPOINT3 *cp3,					//!< Set of CTRLPOINT3's
	MAT3X3 forward,							//!< Forward transformation passed / returned
	MAT3X3 inverse,							//!< Inverse transformation passed / returned
	UINT32 flags								//!< Flags
	);

//! Perform affine transformation on a point
GEOMLIBEXPORT void trans2d (
	double ix,									//!< Input X coordinate
	double iy,									//!< Input Y coordinate
	const MAT3X3 ItoO,						//!< Input to Output transformation
	double *ox,									//!< Output X coordinate
	double *oy									//!< Output Y coordinate
	);

//! Find the inverse transformation
GEOMLIBEXPORT void trans2dfindinverse (
	const MAT3X3 fwd,							//!< Input transformation
	MAT3X3 inv									//!< Output inverse transformation
	);

//! Initialize an affine transformation
//!
//! This will initialize the transformation to "identity".
GEOMLIBEXPORT void trans2dinit (
	MAT3X3 ItoO,								//!< Input to Output transformation
	MAT3X3 OtoI									//!< Output to Input transformation
	);

//! Shift forward/inverse affine transformation so specified points match
GEOMLIBEXPORT void trans2dmatch (
	MAT3X3 f,									//!< Input to Output transformation
	MAT3X3 i,									//!< Output to Input transformation
	double ix,									//!< Input x
	double iy,									//!< Input y
	double ox,									//!< Output x
	double oy									//!< Output y
	);

//! Apply a matrix to a transformation ** not reversible
GEOMLIBEXPORT void trans2dmatrix (
	MAT3X3 m,									//!< Output matrix
	const MAT3X3 mat							//!< Input matrix
	);

//! Apply a matrix to a transformation ** not reversible
GEOMLIBEXPORT void trans2dmatrix1 (
	const MAT3X3 m,							//!< Input matrices
	const MAT3X3 mat,
	MAT3X3 out									//!< Output matrix
	);

//! Combine two affine transformations (both forward and inverse)
GEOMLIBEXPORT void trans2dmatrix2 (
	MAT3X3 atoc,								//!< Output matrices
	MAT3X3 ctoa,
	const MAT3X3 atob,						//!< Input matrices
	const MAT3X3 btoa,
	const MAT3X3 btoc,
	const MAT3X3 ctob
	);

//! Rotate by ang (radians) about point x y
GEOMLIBEXPORT void trans2drotpt (
	MAT3X3 fwd,									//!< Forward transformation
	MAT3X3 inv,									//!< Inverse transformation
	double angle,								//!< Angle to rotate by
	double ptx,									//!< Point to rotat about
	double pty
	);

//! Apply a rotation to an affine transformation
GEOMLIBEXPORT void trans2drot (
	MAT3X3 m,									//!< Input to Output transformation
	MAT3X3 n,									//!< Output to Input transformation
	double angle								//!< Angle to rotate by
	);

//! Scale image by sx, sy
GEOMLIBEXPORT void trans2dscale (
	MAT3X3 m,									//!< Input to Output transformation
	MAT3X3 n,									//!< Output to Input transformation
	double sx,									//!< X scale
	double sy									//!< Y scale
	);

//! Apply a shear to an affine transformation
GEOMLIBEXPORT void trans2dshear (
	MAT3X3 m,									//!< Input to Output transformation
	MAT3X3 n,									//!< Output to Input transformation
	double shear,								//!< Shear angle in radians (0 = no shear)
	int axis										//!< Axis to shear (0 = X, 1 = Y)
	);

//! Apply an offset to an affine transformation
GEOMLIBEXPORT void trans2dshift (
	MAT3X3 m,									//!< Input to Output transformation
	MAT3X3 n,									//!< Output to Input transformation
	double dx,									//!< X shift
	double dy									//!< Y shift
	);

#define	trans2dorient(m)	SIGN(m[0][0]*m[1][1]-m[1][0]*m[0][1])

//! Apply an affine transformation to a DRECTXY
GEOMLIBEXPORT void Trans2DRect (
	const MAT3X3 trans,						//!< Affine transformation
	const DRECT2D *irect,					//!< Input rectangle
	DRECT2D *orect								//!< Outpu rectange
	);

#if defined(__cplusplus)
}
#endif

#endif					// #ifndef INC_MI32_TRANS2D_H 
