/**
 * \file rvc/spotobj.h
 * \brief interface for the RVC::SPOTOBJ class.
 *
 * \if NODOC
 * $Id: spotobj.h_v 1.2 2005/12/09 17:29:58 scowan Exp $
 *
 * $Log: spotobj.h_v $
 * Revision 1.2  2005/12/09 17:29:58  scowan
 * Updated to use credentials.
 *
 * Revision 1.1  2003/12/26 22:20:32  scowan
 * Initial revision
 *
 *
 * \endif
**/

//!
//!	\class RVC::SPOTOBJ rvc/spotobj.h
//!	\brief Manages RVC::OBJTYPE_SpotHeader objects
//!	
//!	RVC Object Types that this class supports: 
//!		- RVC::OBJTYPE_SpotHeader
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_SpotHeader
//!
//!	Valid parents for this class:
//!		- OBJTYPE_Raster?

#ifndef INC_RVC_SPOTOBJ_H
#define INC_RVC_SPOTOBJ_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

struct CTRLPOINT3;

namespace RVC {

class SPOTHANDLE;

class SPOTOBJ : public RVC::OBJECT {
	public:

		//! Angular rate structure.
		struct ANGULARRATE {
			int line;									//!< Image line number
			int yaw_axis;								//!< Residual angular rate about yaw axis
			int roll_axis;								//!< Residual angular rate about roll axis
			int pitch_axis;							//!< Residual angular rate about pitch axis
			};

		//! SPOT angular info structure.
		struct SPOTANGULARINFO {
			ANGULARRATE *Rate;						//!< Angular rate structure
			int NumElem;								//!< Number of sets in structure (72 or 73 depending on scene)
			};

		//! Default constructor
		SPOTOBJ (
			);

		//! Copy constructor
		SPOTOBJ (
			const RVC::SPOTOBJ& obj
			);

		//! Destructor
		virtual ~SPOTOBJ (
			);

		//! Assignment
		RVC::SPOTOBJ& operator= (
			const RVC::SPOTOBJ& rhs
			);

		//! Get K,J coordinates in GRS (Grid Reference System).
		ERRVALUE GetGRSIdentification (
			int& K,										//!< K-coordinate returned
			int& J										//!< J-coordinate returned
			);

		//! Get Lat/Lon positions of the scene center.
		ERRVALUE GetCenterLatLonPosition (
			CTRLPOINT3& Center						//!< Center position returned
			);

		//! Get Lat/Lon coordinates for the scene corners.
		ERRVALUE GetSceneCorners (
			CTRLPOINT3 Corner[4]						//!< Position of the the 4 scene corners
			);

		//! Get angle through which the image centerline direction must be rotated in order to align with true east at the center of the raw image.
		//!
		//! NOTE: The angle given is the 90 degrees complement of the angle
		//! between the lines of the raw image and true north. the convention of the
		//! sign of the scene orientation angle is given in Appendix 7 of the
		//! SPOT manual.
		ERRVALUE GetImageOrientation (
			double& Angle								//!< Angle returned (in degress)
			);

		//! Get angle between the look direction at the scene center and a line perpendicular to the reference ellipsoid.
		//!
		//! NOTE: sign of the angle:
		//! < 0 (Left) - indicates that satellite passed to the
		//! 	west of the scene center when the scene was acquired.
		//! > 0 (Right) - it passed to the east of the scene center
		ERRVALUE GetAngleOfIncidence (
			double& Angle								//!< Angle returned in degrees
			);

		//! Get sun azimuth and elevation aangles at the scene center at the moment the center pixel is acquired as indicated in the "Viewing date".
		//!
		//! (see GetViewingDate)
		ERRVALUE GetSunAngles (
			double& Azimuth,							//!< Azimuth angle returned in degrees
			double& Elevation							//!< Elevation angle returned in degrees
			);

		//! Get date and time of the scene acquisition.
		ERRVALUE GetViewingDate (
			DATETIME& ViewDate
			);

		//! Get name and number of the satellite (for example SPOT2).
		ERRVALUE GetSatelliteID (
			MISTRING& Name									//!< Satellite name returned
			);

		//! Get name of the instrument used to acquire scene in question.
		ERRVALUE GetInstrumentName (
			MISTRING& Name									//!< Instrument name returned ("HRV1" or "HRV2")
			);

		//! Get spectral mode.
		ERRVALUE GetSpectralMode (
			MISTRING& ModeName							//!< Mode name ("XS" for multispectral or "PAN" for panchromatic )
			);

		//! Get spectral mode P-code.
		ERRVALUE GetSpectralModePCode (
			MISTRING& PCode								//!< Mode P-code
			);

		//! Get chronological number R of the revolution corresponding to the scene acquisition inside 26-day cycle.
		ERRVALUE GetRevolutionNumber (
			int& Rev										//!< Number of revolutions returned
			);

		//! Get number of lines and columns.
		ERRVALUE GetNumLinesColumns (
			int& numlins,								//!< Number of lines returned
			int& numcols								//!< Number of columns returned
			);

		//! Get preprocessing level.
		ERRVALUE GetPreprocessingLevel (
			MISTRING& Name									//!< Name of the level (1A, 1B, 2A, 2B, S1, S2)
			);

		//! Get map projection used in the case of levels 2 or S2.
		ERRVALUE GetMapProjection (
			MISTRING& Name									//!< Map projection name (up to 32 characters)
			);

		//! Get map coordinate origin:map coordinates of the first pixel on the first line in case of level 2B.
		ERRVALUE GetMapOrigin (
			DPOINT2D& Pos								//!< Coordinates (X,Y) in meters
			);

		//! Get absolute calibration coefficients for computing the equivalent radiance of target  objects from pixel brightness counts.
		//!
		//! NOTE: For panchromatic images, this only A1 coefficients exists., for
		//! multispectral coefficients realte to spectral bands XS1, XS2, XS3 respectively.
		//! The mathematical relation used to compute radiance is:
		//! 	L(k) = x(k) / A(k);
		//! Where: L(k) is the SPOT radiance reqiured
		//! 	x(k) is a pixel brightness count
		//! 	A(k) is a absolute calibration coefficient corresponding to spectral band k.
		//! The absolute calibration coefficients A(k) are used to compesate for the difference in
		//! sensitivity of two instruments which could otherwise result in a slight difference 
		//! in the mean pixel brightness counts of two images in a twin pair.
		ERRVALUE GetCalibration (
			double& A1,									//!< Calibration for 1st channel
			double& A2,									//!< Calibration for 2nd channel (meaningless for panchromatic)
			double& A3									//!< Calibration for 3rd channel (meaningless for panchromatic)
			);

		//! Get satellite ephemeris data at the time of acquisition.
		//!
		//! WARNING: depending upon the date and time of the scene center there are
		//! 8 or 9 sets of data (if there are 8 sets, nineth set has meaningless data)
		//! Ephemeris data are provided for nine points spanning scene acquisition, 
		//! the points being separated by intervals of one minute. 
		//! The X,Y,Z position is given relative to a reference frame defined as folows:
		//! - Z axis is the north direction of the Earth's polar axis
		//! - X axis is in equatorial plane and passes through the Greenwich meridian.
		//! - Y axis is in the equatorial plane and perpendicular to the X-axis
		//! - The 3 axes form a right hand reference system.
		//! Velocity vectors Vx, Vy, Vz are given relative to the same reference frame.
		ERRVALUE GetEphemeris (
			DPOINT3D Pos[9],							//!< 9 sets of satellite coordinates
			DPOINT3D Velocity[9],					//!< 9 sets of veclocity data at these points
			INT32 JulianDay[9],						//!< 9 date parameters (Julian day number since Jan 1 1950)
			INT32 Seconds[9]							//!< 9 time parameters (time of the day in seconds)
			);

		//! Get Julian Date and Time of the scene center.
		ERRVALUE GetJulianDate (
			int& day,									//!< Day number since 01/01/1950
			double& seconds							//!< Seconds in the day
			);

		//! Get satellite angular rates during scene acquisition.
		//!
		//! NOTE: The SPOT bus measures angular rates once every 125 ms
		//! These rates are supplied to enable users who so desire to make
		//! certain corrections for themselves (notably for stereoplotting).
		ERRVALUE GetAngularRates (
			SPOTANGULARINFO& Info					//!< Data returned
			);

		//! Get look direction of the CCD detectors corresponding to the first and last pixels of the first and last lines.
		//!
		//! NOTE: HRV look angles are defined relative to the attitude reference frame
		//! The first angle gives look direction rotation about the pitch axis, 
		//! the second about roll axis. The first two angles represent represent
		//! pitch angles for first and last pixels, the second two angles -
		//! roll angles for the first and last pixels.
		//! For a level 1A image the look angles corresponding to apixel of any position may be
		//! obtained by linear interpolation from the above values.
		ERRVALUE GetLookDirections (
			double Angles[4]							//!< Orientation angles
			);

		//! Get histogram for the channels.
		ERRVALUE GetHistogram (
			int Num,										//!< Channel number (0,1, or 2) for panchromatic images only first exists)
			INT32 H[256]								//!< Histogram data
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);
		virtual bool v_ValidateObjectType (const OBJTYPE otype) const;

		SPOTHANDLE* m_SpotHandle;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_SPOTOBJ_H
