/**
 * \file rvc/stylepatternsymbol.h 
 * \brief interface for the RVC::STYLEPATTERN_SYMBOL class.
 *
 * \if NODOC
 * $Id: stylepatternsymbol.h_v 1.5 2007/04/04 17:49:30 scowan Exp $
 *
 * $Log: stylepatternsymbol.h_v $
 * Revision 1.5  2007/04/04 17:49:30  scowan
 * Added virtual methods.
 *
 * Revision 1.4  2006/09/21 14:19:58  scowan
 * Added pattern type to the base.
 *
 * Revision 1.3  2006/09/20 17:36:08  scowan
 * Fwd decl polyline and polylinelist.
 *
 * Revision 1.2  2006/09/20 16:53:37  dwilliss
 * Added ctor from raw buffer
 *
 * Revision 1.1  2006/09/18 17:54:41  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_STYLEPATTERNSYMBOL_H
#define INC_RVC_STYLEPATTERNSYMBOL_H

#ifndef INC_RVC_STYLEPATTERNBASE_H
	#include <rvc/stylepatternbase.h>
#endif

#ifndef INC_MI32_MG2ENUMS_H
	#include <mi32/mg2enums.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
class PLANECONIC;
class POLYLINE;
class POLYLINELIST;
class SUBREGION2D;
struct TEXTSTYLE;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {

//! Point symbol pattern class.
class STYLEPATTERN_SYMBOL : public STYLEPATTERN_BASE {
	public:

		enum ELEMENT_TYPE {
			ELEMENT_TYPE_Null =			0,
			ELEMENT_TYPE_Circle =		1,
			ELEMENT_TYPE_Arc =			2,
			ELEMENT_TYPE_ArcWedge =		3,
			ELEMENT_TYPE_ArcChord =		4,
			ELEMENT_TYPE_Ellipse =		5,
			ELEMENT_TYPE_EArc =			6,
			ELEMENT_TYPE_EArcWedge =	7,
			ELEMENT_TYPE_EArcChord =	8,
			ELEMENT_TYPE_Line =			9,
			ELEMENT_TYPE_Polygon =		10,
			ELEMENT_TYPE_Point =		11,
			ELEMENT_TYPE_Text =			12,
			ELEMENT_TYPE_Segment =		13,
			ELEMENT_TYPE_Rectangle =	14,
			ELEMENT_TYPE_Polygons =		15,
			ELEMENT_TYPE_Splines	 =	16,
			ELEMENT_TYPE_COUNT
			};

		class ITERATOR;
		class ELEMENT;

		//! Pattern element referencing shared memory buffer.
		//! Primarily used by ITERATOR.
		class ELEMENT_SHARED {
			public:

				//! Get cap style.
				//! @return Line cap style.
				MGD::LINECAP GetCapStyle (
					) const;

				//! Get color index.
				//! @return Index into color array.
				UINT8 GetColorIndex (
					) const { return (m_pData->colornum); }

				MGD::LINEJOIN GetJoinStyle (
					) const;

				//! Get element line thickness.
				//! @return Thickness in 1/100 millimeter.
				INT16 GetThickness (
					) const { return (m_pData->thickness); }

				//! Get element type.
				//! @return Element type.
				ELEMENT_TYPE GetType (
					) const { return (static_cast<ELEMENT_TYPE>(m_pData->type)); }

				//! Determine if element is 'filled'.
				//! @return 'true' if filled, 'false' if not.
				bool IsFilled (
					) const { return ((m_pData->flags & SYMBOLELEM::FLAG_Filled) != 0); }

				//! Get an element as a point.
				//! @return 'true' if convertable to a point.
				bool GetPoint (
					DPOINT2D& point
					) const;

				//! Get an element as a point.
				//! @return 'true' if convertable to a point.
				bool GetPoint (
					FPOINT2D& point
					) const;

				//! Get an element as a point.
				//! @return 'true' if convertable to a point.
				bool GetPoint (
					WPOINT2D& point
					) const;

				//! Get an element as points.
				//! @return 'true' if convertable to points.
				bool GetPoints (
					POLYLINE& points
					) const;
	
				//! Get an element as lines list.
				//! @return 'true' if convertable to lines list.
				bool GetLines (
					POLYLINELIST& lines
					) const;
	
				//! Get an element as polyline.
				//! @return 'true' if convertable to polyline.
				bool GetLine (
					POLYLINE& polyline
					) const;
	
				//! Get an element as a planeconic.
				//! @return 'true' if convertable to a planeconic.
				bool GetPlaneConic (
					PLANECONIC& planeconic
					) const;
	
				//! Get an element as polygon.
				//! @return 'true' if convertable to polygon.
				bool GetPolygon (
					POLYLINE& polygon
					) const;

				//! Get an element as a subregion.
				//! @return 'true' if convertable to a subregion.
				bool GetSubRegion (
					SUBREGION2D& subregion
					) const;
	
				//! Get an element as a text.
				//! @return 'true' if convertable to a text.
				bool GetText (
					MISTRING& string
					) const;
	
				//! Get an element as a text.
				//! @return nonzero pointer if convertable to a text.
				const MIUNICODE* const GetText (
					) const;

				//! Get textstyle for text element.
				//! @return 'true' if convertable to a textstyle.
				bool GetTextStyle (
					TEXTSTYLE& style
					) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				union SYMBOLDATA {
					struct { INT16 x, y, radius; } circle;
					struct { INT16 x, y, radius, sangle, eangle; } arc;
					struct { INT16 x, y, xsize, ysize, rotate; } ellipse;
					struct { INT16 x, y, xsize, ysize, sangle, eangle, rotate; } earc;
					struct { INT16 x, y; } point;
					struct { INT16 x1, y1, x2, y2; UINT16 capstyle; } segment;
					struct { INT16 x, y, xsize, ysize, rotate, shear; UINT16 joinstyle; } rectangle;
					struct { UINT16 numpoints, joinstyle, capstyle; } line;
					struct { UINT16 numpoints, joinstyle; } polygon;
					struct { UINT16 numchars, fontnum, fontstyle, flags;
								INT16 x, y, height, rotate, shear, textrot, xscale; } text;
					struct { UINT16 numpoints, joinstyle, capstyle; } spline;
					struct { UINT16 numpoints, joinstyle, capstyle; } ttfsymbol;
					struct { UINT16 numpoints, joinstyle, numpolys; } polygons;
					INT16 spare[12];
					};

				//! Symbol element structure.
				struct SYMBOLELEM {
					enum FLAGS {
						FLAG_None = 0x0000,
						FLAG_Filled = 0x0001	//!< Fill polygonal elements
						};
					
					SYMBOLDATA data;							//!< Union of element structures
					UINT32 numbytes;							//!< Number of bytes
					INT16 thickness;							//!< Thickness of element lines (if not filled)
					UINT16 flags;								//!< Flags
					UINT8 type;									//!< Type of element stored
					UINT8 colornum;							//!< Color index in pattern-specific color map
					UINT8 spare[2];							//!< Double alignment
					};

				const SYMBOLELEM *m_pData;
				void *m_pExtra;

				//! Default constructor.
				ELEMENT_SHARED (
					): m_pData(0), m_pExtra(0) { }

				explicit ELEMENT_SHARED (const void *pElemData)
					: m_pData(static_cast<const SYMBOLELEM*>(pElemData)), m_pExtra(static_cast<UINT8*>(const_cast<void*>(pElemData)) + sizeof(SYMBOLELEM))
					{ }

				//! Destructor.
				~ELEMENT_SHARED (
					) { }

				friend class STYLEPATTERN_SYMBOL;
				friend class ITERATOR;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Iterator to process pattern elements.
		class ITERATOR {
			public:

				//! Default constructor.
				ITERATOR (
					): m_pEnd(0) { }

				//! Destructor.
				~ITERATOR (
					) { }

				//! Dereference operator, returns reference to ELEMENT_SHARED.
				const ELEMENT_SHARED& operator* (
					) const { return (m_element); }

				//! Dereference operator, returns pointer to ELEMENT_SHARED.
				const ELEMENT_SHARED* operator-> (
					) const { return (&m_element); }

				//! Pre-increment operator.
				ITERATOR& operator++ ();

				//! Equality operator
				bool operator== (
					const ITERATOR& rhs
					) const { return ((m_pEnd == 0 && rhs.m_pEnd == 0) || m_element.m_pData == rhs.m_element.m_pData); }

				//! Inequality operator
				bool operator!= (
					const ITERATOR& rhs
					) const { return (!(*this == rhs)); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				ELEMENT_SHARED m_element;
				const void *m_pEnd;

				ITERATOR (const void *pData, const void *pEnd)
					: m_element(pData), m_pEnd(pEnd)
					{ }

				friend class STYLEPATTERN_SYMBOL;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};
			
		STYLEPATTERN_SYMBOL (
			) : STYLEPATTERN_BASE(PATTERNTYPE_Symbol, sizeof(PATTERNHEADER_SYMBOL)) {}
		
		//! Copy constructor.
		STYLEPATTERN_SYMBOL (
			const STYLEPATTERN_SYMBOL& rhs
			) : STYLEPATTERN_BASE(rhs) {}

		//! Constructor from raw pattern data
		STYLEPATTERN_SYMBOL (
			const void* buf,
			INT32 NumBytes
			) : STYLEPATTERN_BASE(PATTERNTYPE_Symbol, sizeof(PATTERNHEADER_SYMBOL)) {
			SetPattern(buf, NumBytes, sizeof(PATTERNHEADER_SYMBOL));
			}
		
		//! Destructor.
		virtual ~STYLEPATTERN_SYMBOL (
			) { }

		//! Assignment.
		STYLEPATTERN_SYMBOL& operator= (
			const STYLEPATTERN_SYMBOL& rhs
			) { if (this != &rhs) { STYLEPATTERN_BASE::operator=(rhs); } return (*this); }

		//! Get iterator for beginning element.
		ITERATOR Begin (
			) const { return (ITERATOR(GetFirstElemPtr(),GetPattBuffer()+GetSize())); }

		//! Get iterator for end comparison.
		ITERATOR End (
		#ifdef WIN32
			) const { return (ITERATOR()); }
		#else
			) const { ITERATOR it; return (it); }
		#endif

		INT16 GetHotSpotX (
			) const { return (reinterpret_cast<const PATTERNHEADER_SYMBOL*>(m_PattBuff)->xhot); }

		INT16 GetHotSpotY (
			) const { return (reinterpret_cast<const PATTERNHEADER_SYMBOL*>(m_PattBuff)->yhot); }

		INT16 GetSizeX (
			) const { return (reinterpret_cast<const PATTERNHEADER_SYMBOL*>(m_PattBuff)->xsize); }

		INT16 GetSizeY (
			) const { return (reinterpret_cast<const PATTERNHEADER_SYMBOL*>(m_PattBuff)->ysize); }

		bool ComputeSize (
			INT32& xsize,
			INT32& ysize
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Symbol header.
		struct PATTERNHEADER_SYMBOL : public STYLEPATTERN_BASE::PATTERNHEADER {
			enum FLAGS {
				FLAG_None = 0x0000,
				FLAG_OldSymbol = 0x8000
				};
				
			INT16 xsize;								//!< X size
			INT16 ysize;								//!< Y size
			INT16 xhot;									//!< X hot
			INT16 yhot;									//!< Y hot
			UINT16 flags;								//!< Flags
			UINT8 spare[2];							//!< Double alignment
			};
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_STYLEPATTERNSYMBOL_H
