/**
 * \file imagefilter.h <spatmod/imagefilter.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER definition.
 *
 * \if NODOC
 * $Id: imagefilter.h_v 1.18 2011/04/01 12:44:32 mju Exp $
 *
 * $Log: imagefilter.h_v $
 * Revision 1.18  2011/04/01 12:44:32  mju
 * Remove wrong v_logCreate.
 *
 * Revision 1.17  2011/03/31 21:36:16  mju
 * Add v_logCreate.
 *
 * Revision 1.16  2011/03/31 15:23:45  mju
 * *** empty log message ***
 *
 * Revision 1.15  2011/03/31 15:19:44  mju
 * Add v_getTypeName.
 *
 * Revision 1.14  2011/02/03 21:25:42  mju
 * Change v_getGeoreference to use zoom.
 *
 * Revision 1.13  2007/07/16 20:27:50  mju
 * Override v_getPixelLookupTable.
 *
 * Revision 1.12  2007/02/27 19:08:25  mju
 * Add v_getDimensions.
 *
 * Revision 1.11  2007/01/03 23:02:49  mju
 * Add getMetadata.
 *
 * Revision 1.10  2006/12/04 16:17:29  mju
 * Add v_getZoomAbility.
 *
 * Revision 1.9  2006/11/01 23:10:44  mju
 * queryIterator.
 *
 * Revision 1.8  2006/10/16 19:43:06  mju
 * Add class description for doxygen.
 *
 * Revision 1.7  2006/10/16 17:49:47  mju
 * Remove resamplemethod enum to own hdr.
 *
 * Revision 1.6  2006/10/11 14:11:02  mju
 * Use stagearray/autoptr hdrs.
 *
 * Revision 1.5  2005/12/16 16:07:30  mju
 * Add v_getGeoreference.
 *
 * Revision 1.4  2005/09/14 17:30:58  mju
 * Expand docs for v_pullBegin/End.
 *
 * Revision 1.3  2005/08/19 14:25:38  mju
 * Remove setTotalSection.
 *
 * Revision 1.2  2005/08/18 17:16:59  mju
 * Support auto create/destroy for work buffers.
 *
 * Revision 1.1  2005/08/16 15:09:06  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#define	INC_SPATMOD_IMAGEFILTER_H

#ifndef	INC_SPATMOD_IMAGESTAGE_H
#include <spatmod/imagestage.h>
#endif

#ifndef	INC_SPATMOD_IMAGESTAGEARRAY_H
#include <spatmod/imagestagearray.h>
#endif

#ifndef	INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base class for image filter.
class FILTER : public STAGE {
	public:

		// Destructor.
		virtual ~FILTER ();

	protected:

		//! Simple constructor for filter with one previous stage
		//! Does not set any base properties, derived class constructor should do so.
		FILTER (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership						//!< Filter takes ownership of previous stage and will delete when needed
			);

		//! Construct using array of previous stage pointers and common ownership.
		//! Does not set any base properties, derived class constructor should do so.
		FILTER (
			STAGE_ARRAY& PreviousStages,			//!< Array of previous stages
			bool TakeOwnership						//!< Filter takes ownership of previous stages and will delete when needed
			);

		//! Construct with specified number of previous stages.
		//! Does not set any base properties, derived class constructor should do so.
		//! Does not assign any stages, derived class constructor must call SetPreviousStage
		//! for each previous stage.  Used primarily when array of stage pointers is not already
		//! available, eg for derived filter taking fixed number of previous stages.
		FILTER (
			unsigned NumPreviousStages
			);

		//! Create work buffer with specified size using properties from stage index.
		ERRVALUE CreateWorkBuffer (
			UINT32 NumColumns,						//!< Number of columns in buffer
			UINT32 NumRows,							//!< Number of rows in buffer
			unsigned index = 0						//!< Index to stage, 0 for first
			);

		//! Create work buffers for all previous stages with specified size.
		ERRVALUE CreateWorkBuffers (
			UINT32 NumColumns,						//!< Number of columns in buffer
			UINT32 NumRows								//!< Number of rows in buffer
			);

		//! Destroy work buffer for specified input stage if any.
		void DestroyWorkBuffer (
			unsigned index = 0						//!< Index to stage, 0 for first
			);

		//! Destroy all work buffers.
		void DestroyWorkBuffers (
			);

		//! Get number of previous stages.
		unsigned GetNumPreviousStages (
			) const { return (m_PrevStagePtrs.GetNumItems()); }

		//! Get pointer to previous stage in pipeline.
		STAGE* GetPreviousStage (
			unsigned index = 0						//!< Index to stage, 0 for first
			) const { return (m_PrevStagePtrs[index]); }

		//! Get ownership of previous stage in pipeline.
		bool GetPreviousStageOwnership (
			unsigned index = 0						//!< Index to stage, 0 for first
			) const { return (m_PrevStagePtrs[index].IsOwner()); }

		//! Get reference to work buffer for input stage.
		//! The buffer must be created first using CreateWorkBuffer().
		BUFFER& GetWorkBuffer (
			unsigned index = 0						//!< Index to stage, 0 for first
			) const { return (*m_pWorkBuffers[index]); }

		//! Determine if will automatically manage work buffers.
		bool IsAutoCreateWorkBuffers (
			) const { return (m_AutoCreateWorkBuffers); }

		//! Set filter to automatically create work buffers for all input stages in v_PullBegin.
		//! If set, work buffers will be created by v_PullBegin using maximum dimensions from iterator.
		//! By default if this method is not called, work buffers will not be automatically created.
		void SetAutoCreateWorkBuffers (
			bool AutoCreateWorkBuffers = true
			) { m_AutoCreateWorkBuffers = AutoCreateWorkBuffers; }

		//! Set filter to automatically destroy work buffers for all input stages in v_PullEnd.
		//! By default if this method is not called, work buffers will be automatically destroyed.
		void SetAutoDestroyWorkBuffers (
			bool AutoDestroyWorkBuffers
			) { m_AutoDestroyWorkBuffers = AutoDestroyWorkBuffers; }

		//! Set previous stage pointer.
		void SetPreviousStage (
			unsigned index,							//!< Index to stage, 0 for first
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership						//!< Filter takes ownership of previous stage and will delete when needed
			);

		// BASE overrides.

		//! Get dimensions at specified zoom value.
		//! Default implementation returns result of GetDimensions on first input stage.
		virtual ERRVALUE v_GetDimensions (
			DIMENSIONS& dimensions,					//!< Dimensions returned
			double zoom									//!< Zoom/magnification factor
			) const;

		//! Get georeference at specified zoom value.
		//! Default implementation returns georeference from first input stage.
		virtual ERRVALUE v_GetGeoreference (
			GEOREFERENCE& georeference,
			double zoom
			) const;

		//! Get pixel lookup table.
		//! Default implementation returns pointer if set and image has PIXEL::TYPE_Indexed, otherwise calls previous stage method.
		virtual const PIXELTABLE* v_GetPixelLookupTable (
			) const;

		//! Query image for support of specified zoom value.
		//! Default implementation determines zoomability for each input stage and resolves to minimum
		//! allowed set.
		virtual ZOOMABILITY v_GetZoomAbility (
			SIMPLE_ARRAY<double>& ZoomsAllowed	//! Zoom values allowed, interpretation depends on return value
			) const;

		//! Query support/efficiency for specified iterator.
		//! @return Efficiency between 0 and 1, 0 indicates not supported, 1 indicates most efficient.
		//! Default implementation returns product of QueryIterator for each of input pipeline
		//! stages, taking into account differences in tile size if any.  If any source images do not
		//! have the same dimensions as the filter image instance then 0 will always be returned and
		//! derived class must override.
		virtual float v_QueryIterator (
			const SECTION_ITERATOR& iterator		//!< Iterator
			) const;

		// STAGE overrides.

		//! Begin pull of a specific section.
		//! Default implementation calls PullBegin() for earlier stages in pipeline.
		//! If derived class overrides it must either call the base v_PullBegin() with
		//! either the same iterator on one that has been suitably adjusted based on the
		//! filter's behavior, or call PullBegin() for all earlier stages directly.  This
		//! would typically be done before doing any filter-specific processing.
		virtual ERRVALUE v_PullBegin (
			const SECTION_ITERATOR& iterator		//!< SubSection iterator to be used
			);

		//! Finish pulling subsections.
		//! Default implementation calls PullEnd() for earlier stages in pipeline.
		//! If derived class overrides it must either call the base v_PullEnd(), 
		//! or call PullEnd() for all earlier stages directly.  This would typically
		//! be done after doing any filter-specific processing.
		virtual ERRVALUE v_PullEnd (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		STAGE_ARRAY m_PrevStagePtrs;
		BUFFER **m_pWorkBuffers;
		bool m_AutoCreateWorkBuffers;
		bool m_AutoDestroyWorkBuffers;

		void v_GetMetadata (MISTRING& result, METADATACONTENT content, METADATAFORMAT format) const;
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_LogCreate (const char *label, bool OnlyEnds, MILOG::MODE mode);

		// Unimplemented
		FILTER (const FILTER&);
		FILTER& operator= (const FILTER&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTER_H

