/**
 * \file imagefilterfusion.h <spatmod/imagefilterfusion.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_FUSION definition.
 *
 * \if NODOC
 * $Id: imagefilterfusion.h_v 1.6 2007/04/17 15:29:49 mju Exp $
 *
 * $Log: imagefilterfusion.h_v $
 * Revision 1.6  2007/04/17 15:29:49  mju
 * Add highResDoesModulate parm to basichis/hbs ctors.
 *
 * Revision 1.5  2006/12/13 17:31:46  mju
 * Add basichis/hbs filters.
 *
 * Revision 1.4  2006/11/28 16:25:48  mju
 * Add setSaturationMax.
 *
 * Revision 1.3  2006/01/27 22:11:47  mju
 * Create separate implementations for each method.
 *
 * Revision 1.2  2005/08/18 17:17:39  mju
 * No need for pullend override.
 *
 * Revision 1.1  2005/08/16 15:09:39  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERFUSION_H
#define	INC_SPATMOD_IMAGEFILTERFUSION_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

class MIMATRIX;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base Filter to perform fusion of low resolution and high resolution images.
//! If the low resolution image has multiple samples per pixel, the data values should be rescaled
//! or contrast enhanced to similar ranges across all bands to avoid undesirable color shifting, especially for
//! the HIS and HBS methods.
//! The resulting image will have the same dimensions as the high resolution image and the same pixel properties
//! as the low resolution image.  At present the two source images are assumed to have identical geographic extent
//! and projection (eg, one will match the other with simple zooming).
class FILTER_FUSION : public FILTER {
	public:

		// Destructor.
		virtual ~FILTER_FUSION ();

	protected:

		//! Construct using high-resolution and low-resolution source images.
		FILTER_FUSION (
			STAGE *pImageHighRes,					//!< Pointer to high-resolution image
			STAGE *pImageLowRes,						//!< Pointer to low-resolution image
			bool TakeOwnership						//!< Filter takes ownership of previous stages and will delete when needed
			);

		// STAGE/FILTER overrides.

		virtual ERRVALUE v_Initialize ();

		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Unimplemented
		FILTER_FUSION (const FILTER_FUSION&);
		FILTER_FUSION& operator= (const FILTER_FUSION&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

//! Filter to perform fusion of low resolution and high resolution images via "Brovey" method.
class FILTER_FUSION_BROVEY : public FILTER_FUSION {
	public:

		//! Construct using high-resolution and low-resolution source images.
		FILTER_FUSION_BROVEY (
			STAGE *pImageHighRes,					//!< Pointer to high-resolution image
			STAGE *pImageLowRes,						//!< Pointer to low-resolution image
			bool TakeOwnership						//!< Filter takes ownership of previous stages and will delete when needed
			);

		// Destructor.
		virtual ~FILTER_FUSION_BROVEY ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_FUSION_BROVEY (const FILTER_FUSION_BROVEY&);
		FILTER_FUSION_BROVEY& operator= (const FILTER_FUSION_BROVEY&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Filter to perform fusion of low resolution and high resolution images via "Texture PCA" method.
class FILTER_FUSION_TEXTUREPCA : public FILTER_FUSION {
	public:

		//! Construct using high-resolution and low-resolution source images.
		FILTER_FUSION_TEXTUREPCA (
			STAGE *pImageHighRes,					//!< Pointer to high-resolution image
			STAGE *pImageLowRes,						//!< Pointer to low-resolution image
			bool TakeOwnership						//!< Filter takes ownership of previous stages and will delete when needed
			);

		// Destructor.
		virtual ~FILTER_FUSION_TEXTUREPCA ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MIMATRIX *m_pEigenFwdPC;
		MIMATRIX *m_pEigenInvPC;
		double m_Weight;

		ERRVALUE ComputeHighResTexturePCA (const SECTION& TotalSection);

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_FUSION_TEXTUREPCA (const FILTER_FUSION_TEXTUREPCA&);
		FILTER_FUSION_TEXTUREPCA& operator= (const FILTER_FUSION_TEXTUREPCA&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Filter to perform fusion of low resolution and high resolution images via "Modified HIS" method.
class FILTER_FUSION_MODHIS : public FILTER_FUSION {
	public:

		//! Construct using high-resolution and low-resolution source images.
		FILTER_FUSION_MODHIS (
			STAGE *pImageHighRes,					//!< Pointer to high-resolution image
			STAGE *pImageLowRes,						//!< Pointer to low-resolution image
			bool TakeOwnership,						//!< Filter takes ownership of previous stages and will delete when needed
			double MaxHighRes,						//!< HighRes value corresponding to maximum (100%) intensity
			double MaxRGB								//!< Value corresponding to 100% saturation in RGB images.
			);

		// Destructor.
		virtual ~FILTER_FUSION_MODHIS ();

		//! Set proportion of Intensity vs Brightness (HIS vs HBS model)
		void SetIntensityProportion (
			double proportion					//!< Proportion of Intensity vs Brightness (0.0 - 1.0)
			);

		//! Set saturation boost value.
		void SetSaturationBoost (
			double SaturationBoost			//!< Saturation boost (0.5 - 3.0)
			);

		//! Set maximum saturation value.
		//! Default maximum is 80 (percent) if not changed.
		void SetSaturationMax (
			double MaxSaturation				//!< Maximum saturation (percentage, 20-100)
			);

		//! Set whether to perform correction effective color intensity.
		void SetUseEffectiveColorIntensity (
			bool UseEffectiveColorIntensity
			) { m_UseEffectiveColorIntensity = UseEffectiveColorIntensity; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		double m_ScaleBI1;
		double m_ScaleBI2;
		double m_PowSaturationBoost;
		double m_MaxHighRes;
		double m_MaxRGB;
		double m_MaxSaturation;
		double *m_fH;
		double *m_fAA;
		double *m_fAS;
		double *m_baseA;
		double *m_baseS;
		bool m_UseEffectiveColorIntensity;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_FUSION_MODHIS (const FILTER_FUSION_MODHIS&);
		FILTER_FUSION_MODHIS& operator= (const FILTER_FUSION_MODHIS&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Filter to perform fusion of low resolution and high resolution images via "Basic HIS" method.
class FILTER_FUSION_BASICHIS : public FILTER_FUSION {
	public:

		//! Construct using high-resolution and low-resolution source images.
		FILTER_FUSION_BASICHIS (
			STAGE *pImageHighRes,					//!< Pointer to high-resolution image
			STAGE *pImageLowRes,						//!< Pointer to low-resolution image
			bool TakeOwnership,						//!< Filter takes ownership of previous stages and will delete when needed
			double MaxHighRes,						//!< HighRes value corresponding to maximum (100%) intensity
			double MaxRGB,								//!< Value corresponding to 100% saturation in RGB images.
			bool HighResDoesModulate				//!< Use highres to "modulate" intensity rather than replacing
			);

		// Destructor.
		virtual ~FILTER_FUSION_BASICHIS ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		double m_MaxHighRes;
		double m_MaxRGB;
		bool m_HighResDoesModulate;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_FUSION_BASICHIS (const FILTER_FUSION_BASICHIS&);
		FILTER_FUSION_BASICHIS& operator= (const FILTER_FUSION_BASICHIS&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Filter to perform fusion of low resolution and high resolution images via "Basic HBS" method.
class FILTER_FUSION_BASICHBS : public FILTER_FUSION {
	public:

		//! Construct using high-resolution and low-resolution source images.
		FILTER_FUSION_BASICHBS (
			STAGE *pImageHighRes,					//!< Pointer to high-resolution image
			STAGE *pImageLowRes,						//!< Pointer to low-resolution image
			bool TakeOwnership,						//!< Filter takes ownership of previous stages and will delete when needed
			double MaxHighRes,						//!< HighRes value corresponding to maximum (100%) intensity
			double MaxRGB,								//!< Value corresponding to 100% saturation in RGB images.
			bool HighResDoesModulate				//!< Use highres to "modulate" brightness rather than replacing
			);

		// Destructor.
		virtual ~FILTER_FUSION_BASICHBS ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		double m_MaxHighRes;
		double m_MaxRGB;
		bool m_HighResDoesModulate;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_FUSION_BASICHBS (const FILTER_FUSION_BASICHBS&);
		FILTER_FUSION_BASICHBS& operator= (const FILTER_FUSION_BASICHBS&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERFUSION_H

