/**
 * \file imagefiltergeneral.h <spatmod/imagefiltergeneral.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_GENERAL definitions.
 *
 * \if NODOC
 * $Id: imagefiltergeneral.h_v 1.2 2006/06/26 13:22:00 mju Exp $
 *
 * $Log: imagefiltergeneral.h_v $
 * Revision 1.2  2006/06/26 13:22:00  mju
 * Add more docs for delegates.
 *
 * Revision 1.1  2006/05/05 13:08:20  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERGENERAL_H
#define	INC_SPATMOD_IMAGEFILTERGENERAL_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;
class SAMPLEITERATOR;

//=====================================================================================================================

//! General filter using delegate to perform pixel-by-pixel computation with result image properties specified.
//! This filter will call a specified delegate for each pixel to compute the resulting target pixel.  Two variants
//! of this delegate are supported, permitting use of the pixel row and column in computation if required.
class FILTER_GENERAL_CHGPROP : public FILTER {
	public:

		//! Delegate for processing pixel.
		typedef FastDelegate<bool(const SAMPLEITERATOR *itTgt, const SAMPLEITERATOR *itSrc, bool valid)> DELEGATE;

		//! Delegate for processing pixel with row/column parameters.
		typedef FastDelegate<bool(INT32 row, INT32 col, const SAMPLEITERATOR *itTgt, const SAMPLEITERATOR *itSrc, bool valid)> DELEGATE_RC;

		//! Construct with minimal delegate.
		//! The delegate will be passed SAMPLEITERATOR arrays for the target and source, along with a "valid" value indicating
		//! whether the source pixel is valid or not.  Delegate must return true to indicate that result is valid or false if not.
		//! If result is valid delegate must use target iterator to store resulting sample values.
		FILTER_GENERAL_CHGPROP (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const PIXEL& PixelProperties,			//!< Result pixel properties
			DELEGATE delegate							//!< Delegate to call for each pixel
			);

		//! Construct with delegate which accepts row/column parameters.
		//! The delegate will be passed SAMPLEITERATOR arrays for the target and source, along with a "valid" value indicating
		//! whether the source pixel is valid or not.  Delegate must return true to indicate that result is valid or false if not.
		//! If result is valid delegate must use target iterator to store resulting sample values.  In addition, delegate will be
		//! passed the row and column for the image cell being processed.  If these values are not needed then use the minimal
		//! delegate instead for better performance.
		FILTER_GENERAL_CHGPROP (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const PIXEL& PixelProperties,			//!< Result pixel properties
			DELEGATE_RC delegate						//!< Delegate to call for each pixel
			);

		// Destructor.
		virtual ~FILTER_GENERAL_CHGPROP ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE m_Delegate;
		DELEGATE_RC m_DelegateRC;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_GENERAL_CHGPROP (const FILTER_GENERAL_CHGPROP&);
		FILTER_GENERAL_CHGPROP& operator= (const FILTER_GENERAL_CHGPROP&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

//! Filter to perform generic in-place pixel-by-pixel computation using delegate.
//! This filter will call a specified delegate for each pixel to compute the resulting target pixel.  Two variants
//! of this delegate are supported, permitting use of the pixel row and column in computation if required.
class FILTER_GENERAL_INPLACE : public FILTER {
	public:

		//! Delegate for processing each pixel.
		typedef FastDelegate<bool(const SAMPLEITERATOR *it, bool valid)> DELEGATE;

		//! Delegate for processing pixel with row/column parameters.
		typedef FastDelegate<bool(INT32 row, INT32 col, const SAMPLEITERATOR *it, bool valid)> DELEGATE_RC;

		//! Construct with minimal delegate.
		//! The delegate will be passed a SAMPLEITERATOR array, along with a "valid" value indicating whether the source pixel
		//! is valid or not.  Delegate must return true to indicate that result is valid or false if not.
		//! If result is valid delegate must use target iterator to store resulting sample values.
		FILTER_GENERAL_INPLACE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			DELEGATE delegate							//!< Delegate to call for each pixel
			);

		//! Construct with delegate which accepts row/column parameters.
		//! The delegate will be passed a SAMPLEITERATOR array, along with a "valid" value indicating whether the source pixel
		//! is valid or not.  Delegate must return true to indicate that result is valid or false if not.
		//! If result is valid delegate must use target iterator to store resulting sample values.  In addition, delegate will be
		//! passed the row and column for the image cell being processed.  If these values are not needed then use the minimal
		//! delegate instead for better performance.
		FILTER_GENERAL_INPLACE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			DELEGATE_RC delegate						//!< Delegate to call for each pixel
			);

		// Destructor.
		virtual ~FILTER_GENERAL_INPLACE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE m_Delegate;
		DELEGATE_RC m_DelegateRC;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_GENERAL_INPLACE (const FILTER_GENERAL_INPLACE&);
		FILTER_GENERAL_INPLACE& operator= (const FILTER_GENERAL_INPLACE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERGENERAL_H

