/**
 * \file imagefilterpixeltype.h <spatmod/imagefilterpixeltype.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_COLORSPACE definition.
 *
 * \if NODOC
 * $Id: imagefilterpixeltype.h_v 1.3 2012/01/03 22:32:34 bcolombe Exp $
 *
 * $Log: imagefilterpixeltype.h_v $
 * Revision 1.3  2012/01/03 22:32:34  bcolombe
 * Allow converting between gray/grayalpha, rgb/rgba
 *
 * Revision 1.2  2007/12/24 20:54:12  scowan
 * Added methods to determine if conversion is valid without calling initialize.
 *
 * Revision 1.1  2006/08/04 17:12:41  mju
 * Initial revision
 *
 * Revision 1.1  2005/08/16 15:24:45  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERPIXELTYPE_H
#define	INC_SPATMOD_IMAGEFILTERPIXELTYPE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to change pixel type of an image.
//! The following transformations are supported:
//!	Grayscale to RGB, ComplexRI, ComplexMP
//!	RGB to/from CMY, CMYK, IHS, BHS, HSV
//!	CMY to/from CMYK
//!	ComplexRI to/from ComplexMP
//! If source image contains an alpha band it will be passed through unchanged if the target pixel type includes alpha.
//! Output sample types are determined from output pixel type and sample range required.
//! Sample types with predefined ranges exceeding 8-bit unsigned (eg Hue) will have a sample datatype sufficiently large
//! to hold the possible range of sample values.
class FILTER_PIXELTYPE : public FILTER {
	public:

		//! Construct with specified target pixel type.
		FILTER_PIXELTYPE (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			PIXEL::TYPE TargetPixelType,			//!< Target pixel type
			double ColorMaxValue,					//!< Maximum value for color level (R, G, B, C, M, Y, K)
			DATATYPE TargetDatatype = DATATYPE_Invalid	//!< Target data type, default derive from source and ColorMaxValue
			);

		//! Destructor.
		virtual ~FILTER_PIXELTYPE ();
		
		//! Determine if this filter will support the conversion between types.
		//! Can be used to set UI components without initializing the filter.
		static bool IsConversionValid (
			PIXEL::TYPE SourcePixelType, 			//!< Source pixel type
			PIXEL::TYPE TargetPixelType			//!< Target pixel type
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum MODE {
			MODE_Invalid,
			MODE_Gray_to_RGB,
			MODE_Gray_to_GrayAlpha,
			MODE_GrayAlpha_to_Gray,
			MODE_Real_to_ComplexRI,
			MODE_Real_to_ComplexMP,
			MODE_RGB_to_CMY,
			MODE_RGB_to_CMYK,
			MODE_RGB_to_IHS,
			MODE_RGB_to_BHS,
			MODE_RGB_to_HSV,
			MODE_RGB_to_RGBA,
			MODE_RGBA_to_RGB,
			MODE_CMY_to_RGB,
			MODE_CMY_to_CMYK,
			MODE_CMYK_to_RGB,
			MODE_CMYK_to_CMY,
			MODE_ComplexRI_to_ComplexMP,
			MODE_ComplexMP_to_ComplexRI,
			MODE_IHS_to_RGB,
			MODE_BHS_to_RGB,
			MODE_HSV_to_RGB,
			};

		PIXEL::TYPE m_TargetPixelType;
		DATATYPE m_TargetDataType;
		MODE m_Mode;
		double m_ColorMaxValue;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Static support methods
		static MODE GetConversionMode (PIXEL::TYPE SourcePixelType, PIXEL::TYPE TargetPixelType);
		
		// Unimplemented
		FILTER_PIXELTYPE (const FILTER_PIXELTYPE&);
		FILTER_PIXELTYPE& operator= (const FILTER_PIXELTYPE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERPIXELTYPE_H

