/**
 * \file spatmod/imagefilterquantize.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_QUANTIZE definition.
 *
 * \if NODOC
 *
 * $Id: imagefilterquantize.h_v 1.2 2008/08/28 13:57:42 mju Exp $
 *
 * $Log: imagefilterquantize.h_v $
 * Revision 1.2  2008/08/28 13:57:42  mju
 * Fix doxygen file ref.
 *
 * Revision 1.1  2006/11/17 16:50:14  mju
 * Initial revision
 *
 * Revision 1.2  2006/11/16 14:26:41  mju
 * Add setContrastParm.
 *
 * Revision 1.1  2006/01/30 16:43:28  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERQUANTIZE_H
#define	INC_SPATMOD_IMAGEFILTERQUANTIZE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

class CONTRASTPARM;

namespace SPATMOD {
namespace IMAGE {

class SAMPLETABLE;

//=====================================================================================================================

//! Filter to quantize samples in image.
//! Resulting samples will be unsigned with type and precision determined by the number of sample table entries.
class FILTER_QUANTIZE : public FILTER {
	public:

		//! Constructor.
		FILTER_QUANTIZE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership						//!< Filter takes ownership of previous stage and will delete when needed
			);

		//! Destructor.
		virtual ~FILTER_QUANTIZE ();

		//! Get sample table.
		const SAMPLETABLE& GetSampleTable (
			UINT16 SampleNum							//!< Sample index
			) const;

		//! Set sample table.
		//! Any component sample for which a table is not provided will be passed through filter unchanged.
		//! Sample values in table must be in numerically ascending sequence.
		//! Sample datatype must match corresponding sample of previous stage.
		//! Table values are considered to be minimum for quantizing to next sample.  For example, if a table
		//! contains a single sample with value of 100, source samples < 100 will have output value of 0 and
		//! values >= 100 will have output value of 1.  Resulting sample data type will be UINT8 with precision of 1.
		ERRVALUE SetSampleTable (
			UINT16 SampleNum,							//!< Sample index
			const SAMPLETABLE& SampleTable		//!< Sample table
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SAMPLETABLE *m_SampleTableArray;

		void ProcessUINT8 (UINT8* pTgt, const UINT8 *pSrc, const SAMPLETABLE& SampleTable, UINT32 NumColumns) const;
		void ProcessSINT8 (UINT8* pTgt, const INT8 *pSrc, const SAMPLETABLE& SampleTable, UINT32 NumColumns) const;
		void ProcessUINT16 (UINT8* pTgt, const UINT16 *pSrc, const SAMPLETABLE& SampleTable, UINT32 NumColumns) const;
		void ProcessSINT16 (UINT8* pTgt, const INT16 *pSrc, const SAMPLETABLE& SampleTable, UINT32 NumColumns) const;
		void ProcessUINT32 (UINT8* pTgt, const UINT32 *pSrc, const SAMPLETABLE& SampleTable, UINT32 NumColumns) const;
		void ProcessSINT32 (UINT8* pTgt, const INT32 *pSrc, const SAMPLETABLE& SampleTable, UINT32 NumColumns) const;
		void ProcessFloat (UINT8* pTgt, const float *pSrc, const SAMPLETABLE& SampleTable, UINT32 NumColumns) const;
		void ProcessDouble (UINT8* pTgt, const double *pSrc, const SAMPLETABLE& SampleTable, UINT32 NumColumns) const;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_QUANTIZE (const FILTER_QUANTIZE&);
		FILTER_QUANTIZE& operator= (const FILTER_QUANTIZE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERQUANTIZE_H

