/**
 * \file imagefilterzoom.h <spatmod/imagefilterzoom.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_ZOOM definition.
 *
 * \if NODOC
 * $Id: imagefilterzoom.h_v 1.13 2011/03/31 16:04:42 mju Exp $
 *
 * $Log: imagefilterzoom.h_v $
 * Revision 1.13  2011/03/31 16:04:42  mju
 * Add v_getTypeName.
 *
 * Revision 1.12  2010/12/23 20:45:35  mju
 * Don't need scope inside class defn.
 *
 * Revision 1.11  2009/12/17 18:18:08  mju
 * Support minimum/maximum methods.
 *
 * Revision 1.10  2009/12/04 21:02:02  mju
 * Add ctor using specified dimensions.
 *
 * Revision 1.9  2009/04/23 17:46:10  mju
 * Add comments re resamplemethod_Average.
 *
 * Revision 1.8  2009/04/23 17:41:56  mju
 * Add doAverage pvt methods.
 *
 * Revision 1.7  2007/02/27 21:59:34  mju
 * Add prevStageDims member.
 *
 * Revision 1.6  2006/11/01 23:11:11  mju
 * queryIterator.
 *
 * Revision 1.5  2006/10/16 17:49:35  mju
 * Use imageresamplemethod hdr.
 *
 * Revision 1.4  2006/02/14 18:29:29  mju
 * Add prevSection to avoid reading through margin twice.
 *
 * Revision 1.3  2005/08/18 17:17:57  mju
 * No need for pullend override.
 *
 * Revision 1.2  2005/08/18 14:05:29  mju
 * Add members for integer zoom/sample.
 *
 * Revision 1.1  2005/08/16 15:24:55  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERZOOM_H
#define	INC_SPATMOD_IMAGEFILTERZOOM_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

#ifndef	INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to zoom image (up or down) by fixed amount.
class FILTER_ZOOM : public FILTER {
	public:

		//! How to use previous stage to help in zooming if supported.
		enum PREVSTAGEUSAGE {
			PREVSTAGEUSAGE_None,						//!< Don't use previous stage zooming capability at all
			PREVSTAGEUSAGE_Exact,					//!< Use previous stage if supports exact zoom value
			PREVSTAGEUSAGE_Nearest,					//!< Use nearest supported zoom value for previous stage and zoom from there
			};

		//! Construct with specified scale.
		//! RESAMPLEMETHOD_Average is only supported for zoom factors resulting in integer sampling and only up to 1/16.
		FILTER_ZOOM (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			double Scale,								//!< Scale, negative values equivalent to 1/abs(scale)
			PREVSTAGEUSAGE PrevStageUsage,		//!< How to use previous stage to help in zooming if supported
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method if zooming not done entirely by previous stage
			);

		//! Construct with scale to match specified reference image.
		//! Zoom values based on reference image are used during construction, so
		//! must have total dimension available.
		//! RESAMPLEMETHOD_Average is only supported for zoom factors resulting in integer sampling and only up to 1/16.
		FILTER_ZOOM (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const BASE& RefImage,					//!< Reference image to match in total dimensions
			PREVSTAGEUSAGE PrevStageUsage,		//!< How to use previous stage to help in zooming if supported
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method if zooming not done entirely by previous stage
			);

		//! Construct with scale to match specified dimensions.
		//! RESAMPLEMETHOD_Average is only supported for zoom factors resulting in integer sampling and only up to 1/16.
		FILTER_ZOOM (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			INT32 NumColumns,							//!< Number of resulting columns
			INT32 NumRows,								//!< Number of resulting rows
			PREVSTAGEUSAGE PrevStageUsage,		//!< How to use previous stage to help in zooming if supported
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method if zooming not done entirely by previous stage
			);

		// Destructor.
		virtual ~FILTER_ZOOM ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_ZoomX;
		double m_ZoomY;
		double m_PrevStageZoom;
		DIMENSIONS m_PrevStageDims;
		RESAMPLEMETHOD m_ResampleMethod;
		PREVSTAGEUSAGE m_PrevStageUsage;
		INT32 m_IntZoom;
		INT32 m_IntSample;
		INT32 m_Margin;
		SECTION m_PrevSection;

		ERRVALUE BeginIteratorPrevStage (
			const SECTION_ITERATOR& Iterator,
			SECTION_ITERATOR& IteratorPrev
			) const;

		ERRVALUE DoAverage (BUFFER& TgtBuffer, const SECTION& SrcSection, BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageUINT8 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageSINT8 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageUINT16 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageSINT16 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageUINT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageSINT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageFLOAT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageFLOAT64 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;

		ERRVALUE DoMaximum (BUFFER& TgtBuffer, const SECTION& SrcSection, BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumUINT8 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumSINT8 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumUINT16 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumSINT16 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumUINT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumSINT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumFLOAT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumFLOAT64 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;

		ERRVALUE DoMinimum (BUFFER& TgtBuffer, const SECTION& SrcSection, BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumUINT8 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumSINT8 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumUINT16 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumSINT16 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumUINT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumSINT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumFLOAT32 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumFLOAT64 (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;

		ERRVALUE DoNearest (BUFFER& TgtBuffer, const SECTION& SrcSection, const BUFFER& SrcBuffer) const;

		// BASE/STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_ZOOM (const FILTER_ZOOM&);
		FILTER_ZOOM& operator= (const FILTER_ZOOM&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERZOOM_H

