/**
 * \file spatmod/imagetargetmgd.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_MGD definitions.
 *
 * \if NODOC
 * $Id: imagetargetmgd.h_v 1.14 2011/10/31 20:52:52 mju Exp $
 *
 * $Log: imagetargetmgd.h_v $
 * Revision 1.14  2011/10/31 20:52:52  mju
 * Add support for multiple frame buffers.
 *
 * Revision 1.13  2011/10/11 14:54:08  mju
 * Remove unneeded hdr.
 *
 * Revision 1.12  2011/02/07 23:19:30  bcolombe
 * Add SetForceAlphaBlend
 *
 * Revision 1.11  2006/12/22 14:33:26  mju
 * Add src/tgtHasAlpha, maskbuf.
 *
 * Revision 1.10  2006/12/21 21:15:10  mju
 * Add m_needMask.
 *
 * Revision 1.9  2006/11/22 14:28:32  mju
 * Add alphaPos member.
 *
 * Revision 1.8  2006/11/09 22:44:45  mju
 * Change attachDevice to return errvalue.
 *
 * Revision 1.7  2006/11/08 21:27:07  mju
 * Add m_pixelBytes.
 *
 * Revision 1.6  2006/11/06 21:39:45  mju
 * Add members for sample remapping.
 *
 * Revision 1.5  2006/11/06 15:51:52  mju
 * Add initialize so can use directly.
 *
 * Revision 1.1  2006/10/11 17:37:13  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETMGD_H
#define	INC_SPATMOD_IMAGETARGETMGD_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

namespace MGD {
	class CONTEXT;
	class DEVICE;
	}

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base for target device.
class TARGET_MGD : public TARGET {
	public:

		//! Construct with specified input stage.
		//! Target does not take ownership of input stage.
		TARGET_MGD (
			STAGE *pInputStage							//!< Stage in pipeline to obtain image data from
			);

		//! Destructor.
		virtual ~TARGET_MGD ();

		//! Detach device, caller is responsible for destroying device returned.
		//! Will return NULL if not device set or attached.
		//! If device is detached Initialize may be called again to create a new device.
		MGD::DEVICE* DetachDevice (
			) { MGD::DEVICE *pDevice = m_pDevice; m_pDevice = 0; return (pDevice); }

		//! Get attached device, caller must not destroy.
		//! Will return NULL if not device set or attached.
		MGD::DEVICE* GetDevice (
			) const { return (m_pDevice); }

		//! Set device.
		//! If used, must be done before Initialize().
		//! Device dimensions must match image dimensions.
		//! ColorSpace of device must match PixelType and DataType of image.
		//! Ownership of device is not transferred and caller must ensure that
		//! device remains valid until TARGET_MGD instance is destroyed.
		ERRVALUE SetDevice (
			MGD::DEVICE *pDevice
			);

		//! Force context to perform alpha blending in PutImage functions
		void SetForceAlphaBlend (bool bForceAlphaBlend) { m_ForceAlphaBlend = bForceAlphaBlend; }

	protected:

		//! Attach device.
		//! Ownership of device is transferred, and device will be destroyed in
		//! TARGET_MGD instance destructor unless DetachDevice is used.
		ERRVALUE AttachDevice (
			MGD::DEVICE *& pDevice						//!< Device to attach, will be set to NULL on successful return
			);

		// BASE overrides.

		virtual ERRVALUE v_Initialize ();

		//! Query support/efficiency for specified stepping order.
		//! Returns 1 for all orders supported by MGD device, otherwise returns 0.
		virtual float v_QueryIterator (
			const SECTION_ITERATOR& iterator		//!< Iterator
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGD::DEVICE *m_pDevice;
		MGD::CONTEXT *m_pContext;
		bool m_IsOwner;
		bool m_SrcHasAlpha;
		bool m_TgtHasAlpha;
		bool m_ForceAlphaBlend;
		bool m_MatchDeviceFrameBuffers;
		int m_TgtAlphaPos;
		int m_TgtPixelBytes;
		mutable SIMPLE_ARRAY<UINT8> m_RowBuf;
		mutable SIMPLE_ARRAY<UINT8> m_MaskBuf;
		SIMPLE_ARRAY<UINT16> m_SampleTrans;
		UINT16 m_SrcSampleIdxAlpha;

		ERRVALUE PushFrame (UINT16 frame, const SECTION& SubSection, const BUFFER& buffer, const UINT16 *SampleTrans, UINT16 SrcSampleIdxAlpha) const;

		// TARGET overrides.
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_PushSubSection (const SECTION& SubSection, const BUFFER& buffer);

		#endif
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETMGD_H
